﻿;============================================================================================================================
;
;============================================================================================================================

; Directory monitoring procedures

Declare.s Uni2Ansi(*uni.l, uLen)                                      ; Get unicode filename from ANSI filename
Declare   GetRDCWInfo(fniOffset)                                      ; Used by thread monitor to get dir or file changes
Declare   RDCW(dir)                                                   ; Thread to monitor directory changes

; Date and time functions

Declare   GetDayOfWeek(Gadget.l)                                      ; Get the day of the week in name form
Declare   SetDate(Windowid.l)                                         ; Set the current date in the title bar
Declare   SetStat(Field.l, Message.s)                                 ; Shortcut to setting text on the status bar
Declare.s AddDateSuffix(Date.s)                                       ; Add the abbreviated date suffix

; Manipulate INI files

Declare.s GetIniVal(IniSection.s, IniKey.s)                           ; Get a value from an INI file
Declare.l SetIniVal(IniSection.s, IniKey.s, IniValue.s)               ; Set a value in an INI file

; Form and object handling

Declare   GetAllChildHandles(Winhandle)                               ; Find the handle to the status bar
Declare.l CreateImageList()                                           ; Create new icon list for ListIconGadget
Declare   ChangeIcon(mygadget.l, myrow.l, mycolumn.l, myicon.l)       ; Change icons in different collumns
Declare   FlushEvents()                                               ; Prevent forms from greying out
Declare   MyBalloonTips(btWindow.l, btGadget.l, btText.s)             ; Personal balloon tooltip window
Declare   SetListIconColumnText(Gadget, index, HeaderText.s)          ; Set the heading of a ListIconGadget (Danilo)
Declare.s GetListIconColumnText(Gadget, index)                        ; Get the heading of a ListIconGadget (Danilo)
Declare   Display()                                                   ; Shorten the amount of typing we have to do in a ListIconGadget

; Miscellaneous functions

Declare   LastLine(Gadget.l)                                          ; Always go to the last line of a list object
Declare   PrintWeb(prompt)                                            ; Print webgadget items with or without a prompt
Declare   WriteError(Modulename.s)                                    ; Write an error log with module that called it

; Functions for SQLite 3 data handling

Declare.s RepQuote(Instring.s)                                        ; Properly replace single quotes with double for SQL passing
Declare.s KillQuote(Instring.s)                                       ; Kill double quotes in strings for display purposes
Declare   CountRecords()                                              ; Return the number of records in a database
Declare   FillCombo(Gadget.l, Query.s)                                ; Fill a combo box with unique data
Declare.s Sremove(InString.s, Direction.s, Length.l)                  ; Remove trailing and leading chars from a string
Declare   AddRecord()                                                 ; Add a new, blank record to the database
Declare   UpdateStatus()                                              ; Go through the database and update the appointment status fields

; El_Choni's sqlite 3 table manipulation code

Declare.l SQL3GetTable(sSQLQuery.s, *Rows, *Cols, lDataBaseHandle.l)  ; Return data from an SQLite table to a linked list

; Testing this procedure

Declare   ProcessCellSelection()                                      ; Check the listicon in the main window and show stringbox if cell selected (srod)
Declare   GetMonitorPath()                                            ; Get the path you are going to monitor
Declare   DeleteRecord()                                              ; Delete a highlighted record
Declare   ShowData()                                                  ; Show all records for a particular category

Enumeration 1
  #Window_appointment
  #Window_printbox
EndEnumeration

#WindowIndex=#PB_Compiler_EnumerationValue

Enumeration 1
  ;Window_appointment
  #Gadget_appointment_fmain
  #Gadget_appointment_lmonitor
  #Gadget_appointment_monitor
  #Gadget_appointment_appointments
  #Gadget_appointment_fcontrol
  #Gadget_appointment_badd
  #Gadget_appointment_bdelete
  #Gadget_appointment_bimport
  #Gadget_appointment_bexport
  #Gadget_appointment_btagall
  #Gadget_appointment_buntagall
  #Gadget_appointment_bprint
  #Gadget_appointment_bconfig
  #Gadget_appointment_bhelp
  #Gadget_appointment_icategory
  #Gadget_appointment_ccategory
  #Gadget_appointment_isearch
  #Gadget_appointment_ssearch
  #Gadget_appointment_bexit

  #Gadget_printbox_fmain
  #Gadget_printbox_appointments
  #Gadget_printbox_fcontrol
  #Gadget_printbox_bprint
  #Gadget_printbox_bpreview
  #Gadget_printbox_bexit
EndEnumeration

#GadgetIndex=#PB_Compiler_EnumerationValue

Enumeration 1
  #StatusBar_appointment
  #StatusBar_appointment_messages =0
  #StatusBar_appointment_records =1
  #StatusBar_printbox
  #StatusBar_printbox_messages =0
EndEnumeration

#StatusBarIndex=#PB_Compiler_EnumerationValue

Enumeration 1
  #Image_appointment_badd
  #Image_appointment_bdelete
  #Image_appointment_bimport
  #Image_appointment_bexport
  #Image_appointment_btagall
  #Image_appointment_buntagall
  #Image_appointment_bprint
  #Image_appointment_bconfig
  #Image_appointment_bhelp
  #Image_appointment_icategory
  #Image_appointment_isearch
  #Image_appointment_bexit
  #Image_printbox_bprint
  #Image_printbox_bpreview
  #Image_printbox_bexit
EndEnumeration

#ImageIndex=#PB_Compiler_EnumerationValue

CatchImage(#Image_appointment_badd,?_OPT_appointment_badd)
CatchImage(#Image_appointment_bdelete,?_OPT_appointment_bdelete)
CatchImage(#Image_appointment_bimport,?_OPT_appointment_bimport)
CatchImage(#Image_appointment_bexport,?_OPT_appointment_bexport)
CatchImage(#Image_appointment_btagall,?_OPT_appointment_btagall)
CatchImage(#Image_appointment_buntagall,?_OPT_appointment_buntagall)
CatchImage(#Image_appointment_bprint,?_OPT_appointment_bprint)
CatchImage(#Image_appointment_bconfig,?_OPT_appointment_bconfig)
CatchImage(#Image_appointment_bhelp,?_OPT_appointment_bhelp)
CatchImage(#Image_appointment_icategory,?_OPT_appointment_icategory)
CatchImage(#Image_appointment_isearch,?_OPT_appointment_isearch)
CatchImage(#Image_appointment_bexit,?_OPT_appointment_bexit)
CatchImage(#Image_printbox_bprint,?_OPT_appointment_bprint)
CatchImage(#Image_printbox_bpreview,?_OPT_printbox_bpreview)
CatchImage(#Image_printbox_bexit,?_OPT_appointment_bexit)

DataSection
  _OPT_appointment_badd:IncludeBinary "Images\add.ico"
  _OPT_appointment_bdelete:IncludeBinary "Images\delete.ico"
  _OPT_appointment_bimport:IncludeBinary "Images\import.ico"
  _OPT_appointment_bexport:IncludeBinary "Images\export.ico"
  _OPT_appointment_btagall:IncludeBinary "Images\tagall.ico"
  _OPT_appointment_buntagall:IncludeBinary "Images\untagall.ico"
  _OPT_appointment_bprint:IncludeBinary "Images\printer.ico"
  _OPT_appointment_bconfig:IncludeBinary "Images\table1.ico"
  _OPT_appointment_bhelp:IncludeBinary "Images\help.ico"
  _OPT_appointment_icategory:IncludeBinary "Images\category16x16.ico"
  _OPT_appointment_isearch:IncludeBinary "Images\find16x16.ico"
  _OPT_appointment_bexit:IncludeBinary "Images\exit.ico"
  _OPT_printbox_bpreview:IncludeBinary "Images\preview.ico"
EndDataSection



Procedure.l Window_printbox()
  If OpenWindow(#Window_printbox,62,73,800,600,#PB_Window_SystemMenu|#PB_Window_MinimizeGadget|#PB_Window_ScreenCentered|#PB_Window_Invisible,"Print appointments")
    Brush.LOGBRUSH\lbColor=16430784
    SetClassLong_(WindowID(#Window_printbox),#GCL_HBRBACKGROUND,CreateBrushIndirect_(Brush))
    If CreateGadgetList(WindowID(#Window_printbox))
      Frame3DGadget(#Gadget_printbox_fmain,0,0,800,520,"")
      WebGadget(#Gadget_printbox_appointments,5,10,790,505,"")
      Frame3DGadget(#Gadget_printbox_fcontrol,0,515,800,65,"")
      ButtonImageGadget(#Gadget_printbox_bprint,5,525,50,50,UseImage(#Image_printbox_bprint))
      ButtonImageGadget(#Gadget_printbox_bpreview,55,525,50,50,UseImage(#Image_printbox_bpreview))
      ButtonImageGadget(#Gadget_printbox_bexit,745,525,50,50,UseImage(#Image_printbox_bexit))
      CreateStatusBar(#StatusBar_printbox,WindowID(#Window_printbox))
        AddStatusBarField(800)
      HideWindow(#Window_printbox,0)
      ProcedureReturn WindowID()
    EndIf
  EndIf
EndProcedure

;==============================================================================================================================
; Print related constants found by Rings
;==============================================================================================================================

#READYSTATE_UNINITIALIZED       = 0       ; Print routine with WebGadget
#READYSTATE_LOADING             = 1
#READYSTATE_LOADED              = 2
#READYSTATE_INTERACTIVE         = 3
#READYSTATE_COMPLETE            = 4
#OLECMDID_PRINT                 = 6
#OLECMDID_PRINTPREVIEW          = 7
#OLECMDEXECOPT_DONTPROMPTUSER   = 2

;==============================================================================================================================
; SQLite 3 related found by El_Choni
;==============================================================================================================================

#SQLITE3_OK                     =   0     ; Successful Result
#SQLITE3_ERROR                  =   1     ; SQL error Or missing database
#SQLITE3_INTERNAL               =   2     ; An internal logic error in SQLite
#SQLITE3_PERM                   =   3     ; Access permission denied
#SQLITE3_ABORT                  =   4     ; Callback routine requested An abort
#SQLITE3_BUSY                   =   5     ; The database file is locked
#SQLITE3_LOCKED                 =   6     ; A table in The database is locked
#SQLITE3_NOMEM                  =   7     ; A malloc() failed
#SQLITE3_READONLY               =   8     ; Attempt To write A readonly database
#SQLITE3_INTERRUPT              =   9     ; Operation terminated by SQLite_Interrupt()
#SQLITE3_IOERR                  =  10     ; Some kind of disk I/O error occurred
#SQLITE3_CORRUPT                =  11     ; The database disk image is malformed
#SQLITE3_NOTFOUND               =  12     ; (internal Only) table Or record not found
#SQLITE3_FULL                   =  13     ; Insertion failed because database is full
#SQLITE3_CANTOPEN               =  14     ; Unable To open The database file
#SQLITE3_PROTOCOL               =  15     ; database lock protocol error
#SQLITE3_EMPTY                  =  16     ; (internal Only) database table is empty
#SQLITE3_SCHEMA                 =  17     ; The database schema changed
#SQLITE3_TOOBIG                 =  18     ; Too much Data For one Row of A table
#SQLITE3_CONStraint             =  19     ; abort due To contraint violation
#SQLITE3_MISMATCH               =  20     ; Data type mismatch
#SQLITE3_MISUSE                 =  21     ; Library used incorrectly
#SQLITE3_NOLFS                  =  22     ; Uses OS features not supported on host
#SQLITE3_AUTH                   =  23     ; Authorization denied
#SQLITE3_ROW                    = 100     ; sqlite_step() has another Row ready
#SQLITE3_DONE                   = 101     ; sqlite_step() has finished executing

;============================================================================================================================
; Constants for use in ReadDirectoryChangesW function Use these flags in the CallFunctionFast(*pRDCW, ...) line
;============================================================================================================================

#FILE_NOTIFY_CHANGE_FILE_NAME   =    1    ; Any file name change in the watched directory Or subtree causes a change notification wait operation To Return. Changes include renaming, creating, Or deleting a file name.
#FILE_NOTIFY_CHANGE_DIR_NAME    =    2    ; Any directory-name change in the watched directory or subtree causes a change notification wait operation to return. Changes include creating or deleting a directory.
#FILE_NOTIFY_CHANGE_ATTRIBUTES  =    4    ; Any attribute change in the watched directory or subtree causes a change notification wait operation to return.
#FILE_NOTIFY_CHANGE_SIZE        =    8    ; Any file-size change in the watched directory or subtree causes a change notification wait operation to return. The operating system detects a change in file size only when the file is written to the disk. For operating systems that use extensive caching, detection occurs only when the cache is sufficiently flushed.
#FILE_NOTIFY_CHANGE_LAST_WRITE  =  $10    ; Any change to the last write-time of files in the watched directory or subtree causes a change notification wait operation to return. The operating system detects a change to the last write-time only when the file is written to the disk. For operating systems that use extensive caching, detection occurs only when the cache is sufficiently flushed.
#FILE_NOTIFY_CHANGE_LAST_ACCESS =  $20    ; Any change to the last access time of files in the watched directory or subtree causes a change notification wait operation to return.
#FILE_NOTIFY_CHANGE_CREATION    =  $40    ; Any change to the creation time of files in the watched directory or subtree causes a change notification wait operation to return.
#FILE_NOTIFY_CHANGE_SECURITY    = $100    ; Any security-descriptor change in the watched directory or subtree causes a change notification wait operation to return.

;============================================================================================================================
; Constants for myFILE_NOTIFY_INFORMATION (*fni) actions
;============================================================================================================================

#FILE_SHARE_DELETE              = 4
#FILE_ACTION_ADDED              = 1
#FILE_ACTION_REMOVED            = 2
#FILE_ACTION_MODIFIED           = 3
#FILE_ACTION_RENAMED_OLD_NAME   = 4
#FILE_ACTION_RENAMED_NEW_NAME   = 5

;============================================================================================================================
; Constants for myFILE_NOTIFY_INFORMATION offsets This is how we access information when more than 1 action occurs for a file in our watched directory
;============================================================================================================================

#ActionOffset                   = 4
#CharLenOffset                  = 8
#FileNameOffset                 = 12

;============================================================================================================================
; Constants for the listicongadget row and column testing (srod)
;============================================================================================================================

#LVM_SUBITEMHITTEST             = #LVM_FIRST + 57
#LVM_GETSUBITEMRECT             = #LVM_FIRST + 56
#EC_RIGHTMARGIN                 = 2

;============================================================================================================================
; Add icons to rows other than the first one in ListIconGadget
;============================================================================================================================

#LVM_SETEXTENDEDLISTVIEWSTYLE   = $1036
#LVS_EX_SUBITEMIMAGES           = $2

;============================================================================================================================
; My personal constants
;============================================================================================================================

#Version                        = "v0.00"                                   ; Program version
#CopyRight                      = "<°)))o><²³  Appointer(c) " + #Version    ; Copyright string
#Database                       = "Appointer " + #Version + ".ptk"          ; Name of the database
#Eol                            = Chr(10) + Chr(13)                         ; End of line marker

;==============================================================================================================================
; Constants for statusbar handling and colouring
;==============================================================================================================================

#SB_SETBKCOLOR                  = $2001 ; Statusbar colour constant

;============================================================================================================================
; Load the arial forn to match list and combo better (srod)
;============================================================================================================================
 
LoadFont(1,"Arial", 9)

;===========================================================================================================================
; ListIconGadget header data structures
;===========================================================================================================================

Structure LVCOLUMN
  mask.l
  fmt.l
  cx.l
  pszText.s
  cchTextMax.l
  iSubItem.l
  iImage.l
  iOrder.l
EndStructure

;============================================================================================================================
; Custom date routine structure
;==============================================================================================================================

Structure DateStructure ; Custom date routine
  Year.w
  Month.w
  DayOfWeek.w
  Day.w
  Hour.w
  Minute.w
  Second.w
  Milliseconds.w
EndStructure

;==============================================================================================================================
; Window data structure
;==============================================================================================================================

Structure windowdata   ; Window structure data
  winhandle.l           ; Main window handle
  statushandle.l        ; Status bar handle
  newstatwidth.l        ; New status bar width
  oldstatwidth.l        ; Old status bar width
  himagelist.l          ; Image list handle for ListIconGadget
  oldlistproc.l         ; Handle to the old listicongadget procedure
  dateicon.l            ; Handles to ListIconGadget icon replacements
  timeicon.l            ; Handles to ListIconGadget icon replacements
  categoryicon.l        ; Handles to ListIconGadget icon replacements
  statusicon.l          ; Handles to ListIconGadget icon replacements
  notesicon.l           ; Handles to ListIconGadget icon replacements
  row.l                 ; ListIconGadget test row
  collumn.l             ; ListIconGadget test collumn
  yellow.l              ; Object colours
  green.l               ; Object colours
  blue.l                ; Object colours
  colour.l              ; Statusbar colour number
EndStructure

;============================================================================================================================
; Program data structure
;============================================================================================================================

Structure programdata  ; Program data structure
  curdir.s              ; Current program startup directory
  aptdir.s              ; Current program startup directory
  dbhandle.l            ; Database handle
  database.s            ; Name of the database
  inifile.s             ; Name of the system configuration file
  progname.s            ; Program name for base filename saves
  progquit.l            ; User quit the program, so set the quit value to allow program to end repeat loop
  numitems.l            ; Always the true number of items in the database
  curline.l             ; Always the current line in the list object
  daynum.l              ; Numerical day of the week mapped to word format
  weekday.s             ; Named day of the week
  threadid.l            ; Id of the monitor thread
  dirhandle.l           ; Handle of the monitor process
  counter.l             ; How many items in the appointment book
  query.s               ; Current database query
  firstrun.l            ; Do this the first time the program is run
  searchtext.s          ; text from the search box
  category.s            ; Category text from the main form combo box
  record.s              ; Currently highlighted record number
  errorlog.s            ; Name of the system error log file
  errorhandle.l         ; Handle tot he error log file
  curdate.l             ; Current system date
EndStructure

;============================================================================================================================
; Structure to hold appointments in, for processing
;============================================================================================================================

Structure appointmentdata
  filename.s
  tarrived.s
  darrived.s
  date.s
  time.s
  category.s
  status.s
  note.s
  record.s
EndStructure

;============================================================================================================================
; FILE_NOTIFY_INFORMATION structure, used for directory monitor
;============================================================================================================================

Structure myFILE_NOTIFY_INFORMATION
  NextEntryOffset.l
  action.l
  FileNameLength.l
  FileName.w[1]
EndStructure

;============================================================================================================================
; New list to hold sorted categories for combo box gadget
;============================================================================================================================

NewList Categories.s()

;============================================================================================================================
; Constants for testing which cell was selected on a listicongadget (srod)
;============================================================================================================================

; PInfo.LVHITTESTINFO
; temp.Point
; rc.rect
;
;===========================================================================================================================
; New list to hold returned sqlite data
;===========================================================================================================================

NewList SqlData.s()

;==============================================================================================================================
; All global variables
;==============================================================================================================================

Global program.programdata, form.windowdata, user.appointmentdata   ; Global structures for static linked lists

Global *pRDCW                                                       ; Pointer to ReadDirectoryChangesW function
Global *fni.myFILE_NOTIFY_INFORMATION                               ; Pointer of myFILE_NOTIFY_INFORMATION structure
Global hDir                                                         ; Handle to our watched directory
Global quit                                                         ; Variable for ending app when #True

Global WebObject.IWebbrowser2                                       ; For the web page printing

Global PInfo.LVHITTESTINFO, temp.Point, rc.rect                     ; Constants for testing which cell was selected on a listicongadget (srod)

Global sqlite3_open.l,sqlite3_exec.l, sqlite3_close.l               ; All global aliaed SqLite variables
Global sqlite3_errmsg.l, sqlite3_get_table.l, sqlite3_free_table.l  ;
Global sqlite3_changes, sqlite3_last_insert_rowid, sqlite3_free     ;

;===========================================================================================================================
; Object colouring in the form callback. I'm using "Yellow" for most of my gadgets
;===========================================================================================================================

form\yellow  = CreateSolidBrush_(16430784)
form\green   = CreateSolidBrush_($7BDF84)
form\blue    = CreateSolidBrush_($E5B91A)

;===========================================================================================================================
; Statusbar colour number
;===========================================================================================================================

form\colour  = RGB($E9, $DA, $87)

;============================================================================================================================
; ListIconGadget testing (srod)
;============================================================================================================================

form\row = -1                                     ; Indicate no cell has yet been selected.

;==============================================================================================================================
; Get current direectory and store it for later
;==============================================================================================================================

program\curdir = Space(512)                                          ; Give the variable enough space

If GetCurrentDirectory_(Len(program\curdir), @program\curdir) <> 0   ; Get the current directory
  If Right(program\curdir, 1) <> "\"                                 ; Each O/S does it differently so check for backspace
    program\curdir + "\"
  EndIf
EndIf

;==============================================================================================================================
; Program name, sqlite library, database and ini file directory
;==============================================================================================================================

program\inifile      = program\curdir + "appointment.ini"  ; Where the program initialisation file resides
program\errorlog     = program\curdir + "appointment.log"  ; Name of the error log file
program\database     = program\curdir + #Database          ; Name and path to the database

;==============================================================================================================================
; Setup day and month literal names
;==============================================================================================================================

Dim NameOfDay.s(7)                                        ; Fill an array with the names of the days (Terry Hough I think)

  NameOfDay(0)      = "Sunday"
  NameOfDay(1)      = "Monday"
  NameOfDay(2)      = "Tuesday"
  NameOfDay(3)      = "Wednesday" 
  NameOfDay(4)      = "Thursday"
  NameOfDay(5)      = "Friday"
  NameOfDay(6)      = "Saturday"

Dim DaysPerMonth(12)                                      ; Fill an array on how many days per month there are

  For X = 0 To 11     
    DaysPerMonth(X) = 31
  Next

  DaysPerMonth(1)   = 28
  DaysPerMonth(3)   = 30
  DaysPerMonth(5)   = 30
  DaysPerMonth(8)   = 30

  DaysPerMonth(10)  = 30

Dim NameOfMonth.s(12)                                    ; fill an array with the names of the months

  NameOfMonth(0)    = "January"
  NameOfMonth(1)    = "February"
  NameOfMonth(2)    = "March"
  NameOfMonth(3)    = "April"
  NameOfMonth(4)    = "May"
  NameOfMonth(5)    = "June"
  NameOfMonth(6)    = "July"
  NameOfMonth(7)    = "August"
  NameOfMonth(8)    = "September"
  NameOfMonth(9)    = "October"
  NameOfMonth(10)   = "November"
  NameOfMonth(11)   = "December"

Dim Years.s(7)                                           ; fill an array with the years

  Years(0)          = "2002"
  Years(1)          = "2003"
  Years(2)          = "2004"
  Years(3)          = "2005"
  Years(4)          = "2006"
  Years(5)          = "2007"
  Years(6)          = "2008"

;==============================================================================================================================
;
;==============================================================================================================================

Enumeration #GadgetIndex
  #Systray_appointment_tray
  #Gadget_appointment_stringbox
  #Shortcut_appointment_exit
  #Shortcut_appointment_enter
  #Shortcut_config_escape
  #Shortcut_printbox_escape
EndEnumeration

;==============================================================================================================================
; List and status bar images
;==============================================================================================================================

Enumeration #ImageIndex        ; Use last image enumeration, continued from purevision created form
  #Image_appointment_messages   ; This is the mssages in the status bar
  #Image_appointment_records    ; This is the number of records in the status bar
  #Image_appointment_tray       ; This is the icon used when the program is in the tray
 
  #Image_appointment_date       ; This is the primary icon used in the ListIconGadget for the date column
  #Image_appointment_time       ; This is the primary icon used in the ListIconGadget for the time column
  #Image_appointment_category   ; This is the primary icon used in the ListIconGadget for the category column
  #Image_appointment_status     ; This is the primary icon used in the ListIconGadget for the status column
  #Image_appointment_notes      ; This is the primary icon used in the ListIconGadget for the notes column
EndEnumeration

;==============================================================================================================================
; Grab the images to memory for use
;==============================================================================================================================

CatchImage(#Image_appointment_messages,   ?_PTK_appointment_messages)
CatchImage(#Image_appointment_records,    ?_PTK_appointment_records)
CatchImage(#Image_appointment_tray,       ?_PTK_appointment_tray)

CatchImage(#Image_appointment_date,       ?_PTK_appointment_date)
CatchImage(#Image_appointment_time,       ?_PTK_appointment_time)
CatchImage(#Image_appointment_category,   ?_PTK_appointment_category)
CatchImage(#Image_appointment_status,     ?_PTK_appointment_status)
CatchImage(#Image_appointment_notes,      ?_PTK_appointment_notes)

;==============================================================================================================================
; All processable data
;==============================================================================================================================

DataSection
  _PTK_appointment_messages   : IncludeBinary "Images\messages.ico"
  _PTK_appointment_records    : IncludeBinary "Images\records.ico"
  _PTK_appointment_tray       : IncludeBinary "Images\tray.ico"
 
  _PTK_appointment_date       : IncludeBinary "Images\date.ico"
  _PTK_appointment_time       : IncludeBinary "Images\time.ico"
  _PTK_appointment_category   : IncludeBinary "Images\category.ico"
  _PTK_appointment_status     : IncludeBinary "Images\status.ico"
  _PTK_appointment_notes      : IncludeBinary "Images\notes.ico"
 
  sqlite3_dll                 : IncludeBinary "Libraries\sqlite3upx.dll"
EndDataSection

;============================================================================================================================
; Procedure to convert Unicode file name to ANSI
;============================================================================================================================

Procedure.s Uni2Ansi(*uni.l, uLen)
  ansi.s = Space(uLen)
  WideCharToMultiByte_(#CP_ACP, 0, *uni, -1, @ansi.s, uLen, #Null, #Null)
  ProcedureReturn ansi.s
EndProcedure

;============================================================================================================================
; Procedure (Thread) to set the ReadDirectoryChangesW calls
;============================================================================================================================

Procedure RDCW(dir)
  *fni = AllocateMemory(1024)            ; Allocate memory for our myFILE_NOTIFY_INFORMATION pointer Loop calls to ReadDirectoryChangesW We're watching for rename, delete, create, write
  While CallFunctionFast(*pRDCW, dir, *fni, 1024, #True, #FILE_NOTIFY_CHANGE_FILE_NAME | #FILE_NOTIFY_CHANGE_DIR_NAME | #FILE_NOTIFY_CHANGE_LAST_WRITE, @size, #Null, #Null)
    GetRDCWInfo(0)                        ; Get the info for offset 0
    nextEntry = *fni\NextEntryOffset      ; See if there are more entries for this call
    While nextEntry > 0                  ; If so, get the info
      If previousEntry = nextEntry
        Break
      EndIf
      GetRDCWInfo(nextEntry)
      nextEntry = PeekL(*fni + nextEntry) ; See if there are more entries for this call
      previousEntry = nextEntry
    Wend
  Wend                                    ; No more entries, go to next ReadDirectoryChangesW call
EndProcedure

;============================================================================================================================
; Find the handle to a specific child on a window. In this case, the status bar.
;============================================================================================================================

Procedure GetAllChildHandles(Winhandle)
  Childhandle = GetWindow_(Winhandle, #GW_CHILD)
  Repeat
    Namespace.s = Space(999)
    GetClassName_(Childhandle, Namespace.s, 999)
    If Namespace.s = "msctls_statusbar32"
      form\statushandle = Childhandle
    EndIf
    Childhandle = GetWindow_(Childhandle, #GW_HWNDNEXT)
  Until Childhandle = 0
EndProcedure

;============================================================================================================================
; Adds a suffix To the End of a <= 31 numeral 'date'
;============================================================================================================================

Procedure.s AddDateSuffix(Date.s)
  If Date = "1" Or Date = "21" Or Date = "31"
    Date = Date + "st"
  ElseIf Date = "2" Or Date = "22"
    Date = Date + "nd"
  ElseIf Date = "3" Or Date = "23"
    Date = Date + "rd"
  Else
    Date = Date + "th"
  EndIf
  ProcedureReturn Date
EndProcedure

;============================================================================================================================
; Get the current date and the day of the week in word form
;============================================================================================================================

Procedure GetDayOfWeek(Gadget.l)
  program\daynum = DayOfWeek(ParseDate("%dd/%mm/%yyyy", GetGadgetText(Gadget.l)))
  Select program\daynum
    Case 0 : program\weekday = "Sunday"
    Case 1 : program\weekday = "Monday"
    Case 2 : program\weekday = "Tuesday"
    Case 3 : program\weekday = "Wednesday"
    Case 4 : program\weekday = "Thursday"
    Case 5 : program\weekday = "Friday"
    Case 6 : program\weekday = "Saturday"
  EndSelect
EndProcedure

;============================================================================================================================
; Sort out the date and display it
;============================================================================================================================

Procedure SetDate(Windowid.l)
  newDate.dateStructure
  GetSystemTime_(@newDate)
  WeekDay.b = newDate\DayOfWeek
  Day.b     = newDate\Day
  Month.b   = newDate\Month
  Year.w    = newDate\Year
  CurrentDate.s = NameOfDay(WeekDay) + ", " + AddDateSuffix(Str(Day)) + ", " + NameOfMonth(Month - 1) + ", " + Str(Year)
  SetWindowTitle(Windowid.l, #CopyRight + "   --   Today is " + CurrentDate.s)
EndProcedure

;============================================================================================================================
; Custom statusbar routine
;============================================================================================================================

Procedure SetStat(Field.l, Message.s)
  StatusBarText(#StatusBar_appointment, Field.l, Message.s, 0)
EndProcedure

;============================================================================================================================
; New ListIcon procedure handler by Srod, customised for my list and combo setup
;============================================================================================================================

Procedure.l ListIconCallBack(hWnd, uMsg, wParam, lParam)  ; The following callback allows a listicon control to be subclassed.
  Protected Result
  Select uMsg
    Case #WM_COMMAND                                      ; This is how the CBN_SELCHANGE message is sent.
      Select (wParam >> 16) & $ffff                       ; This contains the notification code.
        Case #EN_CHANGE
          ChangedData.s = GetGadgetText(#Gadget_appointment_stringbox)
          SetGadgetItemText(#Gadget_appointment_appointments, form\row, ChangedData.s, form\collumn)
          ;------------------------------------------------------------------------------------------------
          ; Always get current line and record number when a cell is selected for my own functions
          ;------------------------------------------------------------------------------------------------
          program\record  = GetGadgetItemText(#Gadget_appointment_appointments, form\row, 5)  ; Col 4 is record number
          ;------------------------------------------------------------------------------------------------
          ; Find out which collumn has changed for the appropriate SQL query
          ;------------------------------------------------------------------------------------------------
          Select form\collumn
            Case 0  : ColName.s = "time"
            Case 1  : ColName.s = "date"
            Case 2  : ColName.s = "category"
            Case 3  : ColName.s = "status"
            Case 4  : ColName.s = "note"
          EndSelect
          ;------------------------------------------------------------------------------------------------
          ; Setup the query text and write it to the database if possible
          ;------------------------------------------------------------------------------------------------
          program\query = "Update appointments Set " + ColName.s + "='" + ChangedData.s + "' Where record='" + program\record + "'"
          If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
            SetStat(#StatusBar_appointment_messages, ColName.s + " field in record " + program\record + " updated")
          Else
            SetStat(#StatusBar_appointment_messages, "Error: The change could not be written back to the database!")
            WriteError("ListIconCallBack")
          EndIf
      EndSelect
  EndSelect
  Result = CallWindowProc_(form\oldlistproc, hWnd, uMsg, wParam, lParam)
  ProcedureReturn Result
EndProcedure

;============================================================================================================================
; Check for NT4/2000/XP/2003 to get the correct version of dll lib
;============================================================================================================================

Select OSVersion()
  Case #PB_OS_Windows_NT_4
    osResult = #True
  Case #PB_OS_Windows_2000
    osResult = #True
  Case #PB_OS_Windows_XP
    osResult = #True
  Case #PB_OS_Windows_Server_2003
    osResult = #True
  Default
    osResult = #False
EndSelect

OperatingSystem.s = "Windows NT4" + #CRLF$ + "Windows 2000" + #CRLF$ + "Windows XP" + #CRLF$ + "Windows Server 2003"

If osResult = #False
  MessageRequester("Incorrect OS", "This program requires one of the following OS's:" + #CRLF$ + OperatingSystem.s)
  End
EndIf

;============================================================================================================================
; Open KERNEL32.DLL for ReadDirectoryChangesW function
;============================================================================================================================

If OpenLibrary(0, "KERNEL32.DLL")
  *pRDCW = IsFunction(0, "ReadDirectoryChangesW") ; Get pointer to ReadDirectoryChangesW function
  If *pRDCW = 0                                   ; If function not found, close KERNEL32.DLL and quit
    CloseLibrary(0)
    MessageRequester("Error", "ReadDirectoryChangesW function not found.")
    End
  EndIf
Else                                              ; If KERNEL32.DLL not found, quit
  MessageRequester("Error", "KERNEL32.DLL not found.")
  End
EndIf

;============================================================================================================================
; Procedure to retrieve our directory changes fniOffset is used for muliple action on 1 file
;============================================================================================================================
 
Procedure GetRDCWInfo(fniOffset)
  action      = PeekL(*fni + #ActionOffset + fniOffset)                  ; Get the event type
  timeStamp.s = FormatDate("%dd/%mm/%yyyy %hh:%ii:%ss", Date())          ; Get a time stamp for this action
  uniLen      = PeekL(*fni + #CharLenOffset + fniOffset)                 ; Get the UNI length of the file name
  fileName.s  = Uni2Ansi(*fni + #FileNameOffset + fniOffset, uniLen / 2) ; Get the ANSI version of the file name
  ;If UCase(GetExtensionPart(Filename$)) = "EXE"                         ; modified to output only executables
    Select action                                                        ; Set the string to be displayed for the action
      Case #FILE_ACTION_ADDED                                            ; A file was added to the monitored directory
      ;----------------------------------------------------------------------------------------------------
      ; These are my specific functions for my appointment monitor but you could do whatever you need to here for files
      ;----------------------------------------------------------------------------------------------------
        Delay(250)                                                       ; Windows needs a slight delay before the file gets noticed
        If ReadFile(0, program\aptdir + fileName.s) <> 0                 ; Open the physical file containing the details
          user\filename = program\aptdir + fileName.s
          user\tarrived = FormatDate("%hh:%ii:%ss",   Date())
          user\darrived = FormatDate("%dd/%mm/%yyyy", Date())
          user\time     = ReadString()
          user\date     = ReadString()
          user\category = ReadString()
          user\status   = ""
          user\note     = ReadString()
          CloseFile(0)
          program\query = "INSERT INTO appointments("
          program\query + "filename,tarrived,darrived,time,date,category,status,note)"
          program\query + "VALUES('" + user\filename  + "','" + user\tarrived + "','" + user\darrived
          program\query + "','"      + user\time      + "','" + user\date     + "','" + user\category
          program\query + "','"      + user\status    + "','" + user\note     + "')"
          If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
            user\record = Str(CallCFunctionFast(sqlite3_last_insert_rowid, program\dbhandle))             ; Last inserted record
            UpdateStatus()
            Display()
            SetStat(#StatusBar_appointment_records, Str(program\counter) + " Record(s)")
            FillCombo(#Gadget_appointment_ccategory, "Select DISTINCT category From appointments ORDER BY category")
          Else
            SetStat(#StatusBar_appointment_messages, "Error: Could not save appointment " + program\aptdir + fileName + " to databse")
          EndIf
        Else
          SetStat(#StatusBar_appointment_messages, "Error: Could not read appointment " + program\aptdir + fileName)
        EndIf
      ;Case #FILE_ACTION_REMOVED
      ;Case #FILE_ACTION_MODIFIED
      ;Case #FILE_ACTION_RENAMED_OLD_NAME
      ;Case #FILE_ACTION_RENAMED_NEW_NAME
    EndSelect
;  EndIf
EndProcedure

;============================================================================================================================
;
;============================================================================================================================

Procedure ChangeIcon(mygadget.l, myrow.l, mycolumn.l, myicon.l)
  LVG.lv_item     
  LVG\mask     = #LVIF_IMAGE | #LVIF_TEXT
  LVG\iItem    = myrow
  LVG\iSubItem = mycolumn
  changetext.s = GetGadgetItemText(mygadget, myrow, mycolumn)
  LVG\pszText  = @changetext.s
  LVG\iImage   = myicon 
  SendMessage_(GadgetID(mygadget), #LVM_SETITEM, 0, @LVG)
EndProcedure

;===========================================================================================================================
; API procedure to replace Mr Skunk's INI file reading routine
;===========================================================================================================================

Procedure.s GetIniVal(IniSection.s, IniKey.s)                                   ; Procedure returns a string
  Empty.s = ""                                                                   ; Default value if nothing found
  ReturnSpace.s = Space(255)                                                     ; Make sure the variable has plenty of room
  IniData = GetPrivateProfileString_(IniSection.s, IniKey.s, Empty.s, @ReturnSpace.s, 255, program\inifile)
  ProcedureReturn ReturnSpace.s                                                ; Return the data to the calling line
EndProcedure

;===========================================================================================================================
; API procedure to replace Mr Skunk's INI file writing routine
;===========================================================================================================================

Procedure.l SetIniVal(IniSection.s, IniKey.s, IniValue.s)
  Result = WritePrivateProfileString_(IniSection, IniKey, IniValue, program\inifile)
  ProcedureReturn Result
EndProcedure

;============================================================================================================================
; Clear the windows event buffer to avoid greying out of forms
;============================================================================================================================

Procedure FlushEvents()
  While WindowEvent()
  Wend
EndProcedure

;============================================================================================================================
; Set the last line of a ListIconGadget
;============================================================================================================================

Procedure LastLine(Gadget.l)
  SendMessage_(GadgetID(Gadget.l), #LVM_ENSUREVISIBLE, CountGadgetItems(Gadget.l) -1,0) ; Make sure the current line is visible
EndProcedure

;============================================================================================================================
; Either print the web page without a prompt from the user or force a preview
;============================================================================================================================

Procedure PrintWeb(prompt)
  If prompt = 0
    WebObject\ExecWB(#OLECMDID_PRINT, #OLECMDEXECOPT_DONTPROMPTUSER, 0, 0)
  Else
    WebObject\ExecWB(#OLECMDID_PRINTPREVIEW, #OLECMDEXECOPT_DONTPROMPTUSER, 0, 0)
  EndIf
EndProcedure

;============================================================================================================================
; Personal balloon tooltip window
;============================================================================================================================

Procedure MyBalloonTips(btWindow.l, btGadget.l, btText.s)
  ToolTipControl = CreateWindowEx_(0, "ToolTips_Class32", "", $D0000000 | $40, 0, 0, 0, 0, WindowID(btWindow), 0, GetModuleHandle_(0), 0)
  SendMessage_(ToolTipControl, 1044, 0, 0)
  SendMessage_(ToolTipControl, 1043, $F3D97A, 0)
  SendMessage_(ToolTipControl, 1048, 0, 180)
  Button.TOOLINFO\cbSize = SizeOf(TOOLINFO)
  Button\uFlags   = $11
  Button\hWnd     = GadgetID(btGadget)
  Button\uId      = GadgetID(btGadget)
  Button\lpszText = @btText
  SendMessage_(ToolTipControl, $0404, 0, Button)
EndProcedure

;===========================================================================================================================
; Set the heading of a ListIconGadget (Danilo)
;===========================================================================================================================

Procedure SetListIconColumnText(Gadget, index, HeaderText.s)
  lvc.LVCOLUMN
  lvc\mask    = #LVCF_TEXT
  lvc\pszText = HeaderText
  SendMessage_(GadgetID(Gadget), #LVM_SETCOLUMN, index, @lvc)
EndProcedure

;===========================================================================================================================
;  Get the heading of a ListIconGadget (Danilo)
;===========================================================================================================================

Procedure.s GetListIconColumnText(Gadget, index)
  lvc.LVCOLUMN
  lvc\mask       = #LVCF_TEXT
  lvc\pszText    = Space(255)
  lvc\cchTextMax = 255
  SendMessage_(GadgetID(Gadget), #LVM_GETCOLUMN, index, @lvc)
  ProcedureReturn PeekS(lvc\pszText)
EndProcedure

;============================================================================================================================
; Count the number of records in an SQLite database
;============================================================================================================================

Procedure CountRecords()
  If SQL3GetTable("Select Count(record) AS totalrecs FROM appointments", @myRows, @myCols, program\dbhandle)
    If CountList(SqlData.s()) <> 0
      LastElement(SqlData.s())
      ProcedureReturn Val(StringField(SqlData.s(), 1, "|"))
    EndIf
  Else
    ProcedureReturn 0
  EndIf 
EndProcedure

;============================================================================================================================
; Fill a combobox gadget SQLite version
;============================================================================================================================

Procedure FillCombo(Gadget.l, Query.s)
  If SQL3GetTable(Query.s, @myRows, @myCols, program\dbhandle)
    If CountList(SqlData.s()) <> 0
      ClearGadgetItemList(Gadget.l)
      AddGadgetItem(Gadget.l, -1, "All")
      ForEach SqlData.s()
        tempdata.s = StringField(SqlData.s(), 1, "|")
        If tempdata.s <> ""
          AddGadgetItem(Gadget.l, -1, tempdata.s)
        EndIf
      Next
    EndIf
  EndIf
EndProcedure

;============================================================================================================================
; Get data back from an SQLite database table and stuff it into a concatenated linked list
;============================================================================================================================

Procedure.l SQL3GetTable(sSQLQuery.s, *Rows, *Cols, lDataBaseHandle.l)
  ClearList(SqlData.s())
  If CallCFunctionFast(sqlite3_get_table, lDataBaseHandle, sSQLQuery, @LResultsPtr, @LRows, @LCols, @ReturnValue) = #SQLITE3_OK
    PokeL(*Rows, LRows)                                     ; return number of rows/columns
    PokeL(*Cols, LCols)
    If LRows > -1 And LCols > 0
      Address.l  = LResultsPtr                              ; copy data into array
      For Row.l  = 1 To LRows
        For Col.l  = 0 To LCols - 1
          tempdata.s + PeekS(PeekL(Address + (((Row * LCols) + Col) * 4))) + "|"
        Next
       AddElement(SqlData.s())
       SqlData.s() = tempdata
       tempdata.s = ""
      Next
    EndIf
    CallCFunctionFast(sqlite3_free_table, LResultsPtr)      ; free table memory
    ProcedureReturn #True
  Else
    WriteError("SQL3GetTable")
    ProcedureReturn #False
  EndIf
EndProcedure 

;============================================================================================================================
; Uncle Berikco's routine to properly replace single quotes with double for SQL passing
;============================================================================================================================

Procedure.s RepQuote(Instring.s)
  For i = 1 To Len(Instring.s)
    If Mid(Instring.s, i, 1) = "'"
      tmp.s = tmp.s + "''"
    Else
      tmp.s = tmp.s + Mid(Instring.s, i, 1)
    EndIf
  Next i
  ProcedureReturn tmp.s
EndProcedure

;============================================================================================================================
; Kill double quotes in strings for display purposes
;============================================================================================================================

Procedure.s KillQuote(Instring.s)
  ProcedureReturn ReplaceString(Instring.s, "''", "'", 1, 1)
EndProcedure

;============================================================================================================================
; Shorten the amount of typing to do in a ListIconGadget
;============================================================================================================================

Procedure Display()
  FlushEvents()
  AddGadgetItem(#Gadget_appointment_appointments, -1, user\time + Chr(10) + user\date + Chr(10) + user\category + Chr(10) + user\status + Chr(10) + user\note + Chr(10) + user\record)
  ChangeIcon(#Gadget_appointment_appointments, program\counter - 1, 0, form\timeicon)
  ChangeIcon(#Gadget_appointment_appointments, program\counter - 1, 1, form\dateicon)
  ChangeIcon(#Gadget_appointment_appointments, program\counter - 1, 2, form\categoryicon)
  ChangeIcon(#Gadget_appointment_appointments, program\counter - 1, 3, form\statusicon)
  ChangeIcon(#Gadget_appointment_appointments, program\counter - 1, 4, form\notesicon)
  LastLine(#Gadget_appointment_appointments)
EndProcedure

;===========================================================================================================================
; Write the errors to an error log file
;===========================================================================================================================

Procedure WriteError(ModuleName.s)
  program\errorhandle = OpenFile(#PB_Any, program\errorlog)       ; Open the error log file
  CallCFunctionFast(sqlite3_errmsg, @ReturnValue)                 ; Get the last sqlite error message
  FileSeek(Lof())                                                 ; Seek to the end of the file
  WriteStringN("Module: " + ModuleName.s + " " + PeekS(ReturnValue))  ; Write the error to the error log plus module name
  CloseFile(program\errorhandle)                                  ; Close the error log
  program\errorhandle = -1                                        ; Reset the handle to negative value
EndProcedure

;============================================================================================================================
; Remove strings from left or right side of input string
;============================================================================================================================

Procedure.s Sremove(InString.s, Direction.s, Length.l)
  StringLen.l = Len(InString.s)
  Select Direction.s
    Case "L"  : ProcedureReturn = Right(InString.s, StringLen - length)
    Case "R"  : ProcedureReturn = Left(InString.s,  StringLen - length)
  EndSelect
EndProcedure

;============================================================================================================================
; Check the listicon in the main window and show stringbox if cell selected (srod)
;============================================================================================================================

Procedure ProcessCellSelection()
  GetCursorPos_(temp)
  MapWindowPoints_(0, GadgetID(#Gadget_appointment_appointments), temp, 1)
  pInfo\pt\x = temp\x
  pInfo\pt\y = temp\y
  SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_SUBITEMHITTEST, 0, pInfo)
  If pInfo\iItem <> -1 ; And pInfo\iSubItem = 1 ; A cell has been selected. but limit it to cell one.
    form\row     = pInfo\iItem
    form\collumn  = pInfo\iSubItem
    rc\top  = pInfo\iSubItem
    rc\left = #LVIR_BOUNDS
    SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_GETSUBITEMRECT, pInfo\iItem, rc)
    rc\right = rc\left + SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_GETCOLUMNWIDTH, pInfo\iSubItem, 0)
    ResizeGadget(#Gadget_appointment_stringbox, rc\left, rc\top - 2, rc\right - rc\left, rc\bottom - rc\top + 2)
    SendMessage_(GadgetID(#Gadget_appointment_stringbox), #CB_SETITEMHEIGHT, - 1, rc\bottom - rc\top - 2)
    SetGadgetText(#Gadget_appointment_stringbox, GetGadgetItemText(#Gadget_appointment_appointments, pInfo\iItem, pInfo\iSubItem))
    HideGadget(#Gadget_appointment_stringbox, 0)  ; Show the string box
  Else
    HideGadget(#Gadget_appointment_stringbox, 1)  ; Hide the string box
  EndIf
EndProcedure

;============================================================================================================================
; Get the path you are going to monitor
;============================================================================================================================

Procedure GetMonitorPath()
  program\aptdir  = PathRequester("Path to monitor", "")
  If program\aptdir <> ""
    SetGadgetText(#Gadget_appointment_monitor, program\aptdir)
    SetIniVal("Monitor Dir", "Directory To Monitor", program\aptdir)
    ;------------------------------------------------------------------------------------------------------
    ; Kill the old thread, if one exists
    ;------------------------------------------------------------------------------------------------------
    If program\dirhandle                      ; Free monitor, thread and objects if they previously existed
      FreeMemory(*fni)                                                            ; Free monitor memory
      KillThread(program\threadid)                                                ; Kill the monitor thread
      CloseHandle_(program\dirhandle)                                             ; Close the monitor object
    EndIf
    ;------------------------------------------------------------------------------------------------------
    ; Create the new thread that monitors the directory now
    ;------------------------------------------------------------------------------------------------------
    program\dirhandle = CreateFile_(program\aptdir, #FILE_LIST_DIRECTORY, #FILE_SHARE_WRITE | #FILE_SHARE_READ | #FILE_SHARE_DELETE, #Null, #OPEN_EXISTING, #FILE_FLAG_BACKUP_SEMANTICS, #Null)
    program\threadid  = CreateThread(@RDCW(), program\dirhandle)
    SetStat(#StatusBar_appointment_messages, "Info: Monitor restarted with new information")
  Else
    MessageRequester("Error", "You didn't select a path to monitor so there is nothing to do!")
  EndIf
EndProcedure

;============================================================================================================================
; Delete a highlighted record
;============================================================================================================================

Procedure DeleteRecord()
  program\curline = GetGadgetState(#Gadget_appointment_appointments)
  If program\curline <> -1
    If MessageRequester("WARNING", "You are about to permanently delete a record " + Chr(10) + Chr(13) + "from the database and it cannot be recovered. " + Chr(10) + Chr(13) + "Are you sure you want To do this?", #MB_ICONEXCLAMATION | #PB_MessageRequester_YesNo) = 6
      program\record = GetGadgetItemText(#Gadget_appointment_appointments, program\curline, 5)
      ;----------------------------------------------------------------------------------------------------
      ; Find he name of the physical appointment file linked to the record
      ;----------------------------------------------------------------------------------------------------
      program\query = "Select filename FROM appointments WHERE record = '" + program\record + "'"
      If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
        If SQL3GetTable(program\query, @myRows, @myCols, program\dbhandle)
          If CountList(SqlData.s()) <> 0
            user\filename = StringField(SqlData.s(), 1, "|")
          EndIf
        EndIf
      EndIf
      ;----------------------------------------------------------------------------------------------------
      ; Delete the record fron the database, remove from the lsit and delete the physical record linked to it
      ;----------------------------------------------------------------------------------------------------
      program\query = "DELETE FROM appointments WHERE record = '" + program\record + "'"
      If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
        SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_DELETEITEM, program\curline, 0) ; Delete it now
        DeleteFile(user\filename)
        SetStat(#StatusBar_appointment_messages, "Information: Appointment number " + Str(program\curline + 1) + " deleted permanently")
        SetStat(#StatusBar_appointment_records, Str(program\numitems) + " Record(s)")
      Else
        ;--------------------------------------------------------------------------------------------------
        ; Write the error to disk with the name of the calling module
        ;--------------------------------------------------------------------------------------------------
        WriteError("DeleteRecord")
        CallCFunctionFast(sqlite3_free, @ReturnValue)
        SetStat(#StatusBar_appointment_messages, "Error: Could not delete the selected appointment!")
      EndIf
    EndIf
  EndIf
  program\query   = ""
  program\curline = -1
EndProcedure

;============================================================================================================================
; Show all records for a particular category
;============================================================================================================================

Procedure ShowData()
  If EventType() = #PB_EventType_RightClick Or program\firstrun       ; Don't want the combobox to jump twice, it simulates a right click
    If program\firstrun = 1
      program\firstrun = 0
    EndIf
    ClearGadgetItemList(#Gadget_appointment_appointments)             ; Clear previous results
    program\category  = GetGadgetText(#Gadget_appointment_ccategory)  ; Get the category choice
    Select program\category
      Case "All" : program\query = "SELECT * FROM appointments"
      Case ""    : program\query = "SELECT * FROM appointments"
      Default    : program\query = "SELECT * FROM appointments WHERE category = '" + program\category + "'"
    EndSelect
    If SQL3GetTable(program\query, @myRows, @myCols, program\dbhandle)
      program\counter = 0
      ForEach SqlData.s()
        program\counter + 1
;       user\filename = StringField(SqlData.s(), 1, "|")
        user\tarrived = StringField(SqlData.s(), 2, "|")
        user\darrived = StringField(SqlData.s(), 3, "|")
        user\time     = StringField(SqlData.s(), 4, "|")
        user\date     = StringField(SqlData.s(), 5, "|")
        user\category = StringField(SqlData.s(), 6, "|")
        user\status   = StringField(SqlData.s(), 7, "|")
        user\note     = StringField(SqlData.s(), 8, "|")
        user\record   = StringField(SqlData.s(), 9, "|")
        UpdateStatus()
        Display()
        SetStat(#StatusBar_appointment_records, Str(program\counter) + " Record(s)")
      Next
      SetStat(#StatusBar_appointment_messages, "Information: Showing all records for category [" + program\category + "]")
      SetStat(#StatusBar_appointment_records,  Str(program\numitems) + " Record(s)")             ; Total records in statusbar
    Else
      SetStat(#StatusBar_appointment_messages, "Error: Could not show the requested category from the database")
      WriteError("ShowData")
    EndIf
  EndIf
EndProcedure

;============================================================================================================================
; Add a new, blank record to the database
;============================================================================================================================

Procedure AddRecord()
  user\filename = ""
  user\tarrived = FormatDate("%hh:%ii:%ss",   Date())
  user\darrived = FormatDate("%dd/%mm/%yyyy", Date())
  user\time     = FormatDate("%hh:%ii:%ss",   Date())
  user\date     = FormatDate("%dd/%mm/%yyyy", Date())
  user\category = ""
  user\status   = ""
  user\note     = ""
  program\query = "INSERT INTO appointments("
  program\query + "filename,tarrived,darrived,time,date,category,status,note)"
  program\query + "VALUES('" + user\filename  + "','" + user\tarrived + "','" + user\darrived
  program\query + "','"      + user\time      + "','" + user\date     + "','" + user\category
  program\query + "','"      + user\status    + "','" + user\note     + "')"
  If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
    user\record = Str(CallCFunctionFast(sqlite3_last_insert_rowid, program\dbhandle))             ; Last inserted record
    UpdateStatus()
    Display()
    SetStat(#StatusBar_appointment_records, Str(program\counter) + " Record(s)")
    FillCombo(#Gadget_appointment_ccategory, "Select DISTINCT category From appointments ORDER BY category")
  Else
    SetStat(#StatusBar_appointment_messages, "Error: Could not save new appointment to database")
  EndIf
EndProcedure

;============================================================================================================================
; Go through the database and update the appointment status fields
;============================================================================================================================

Procedure UpdateStatus()
  TestDate.l = ParseDate("%dd/%mm/%yyyy", user\date)
  If program\curdate > TestDate.l        ; If current date is greater than appointment date
    user\status   = "Missed"
  ElseIf program\curdate < TestDate.l   ; If current date is less than appointment date
    user\status   = "Pending"
  ElseIf program\curdate = TestDate.l   ; If current date is equal to than appointment date
    user\status   = "Due today"
  EndIf
  program\query = "Update appointments Set status='" + user\status + "' Where record='" + user\record + "'"
  If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
  Else
    WriteError("UpdateStatus")
  EndIf
EndProcedure

;============================================================================================================================
;
;============================================================================================================================

; XIncludeFile "Appointments_Mydeclarations.pb"
; XIncludeFile "Appointments_Constants.pb"
; XIncludeFile "Appointments_Windows.pb"
; XIncludeFile "Appointments_Myconstants.pb"
; XIncludeFile "Appointments_Myprocedures.pb"

;============================================================================================================================
; Initialise the SQLite dll, create the datebase and the table if they don't exist
;============================================================================================================================

  sqlite3_lib                 = LoadLibraryM(?sqlite3_dll)  ; Load the library from memory, don't write to disk
 
  If sqlite3_lib
    sqlite3_open              = GetProcAddressM(sqlite3_lib, "sqlite3_open")
    sqlite3_exec              = GetProcAddressM(sqlite3_lib, "sqlite3_exec")
    sqlite3_close             = GetProcAddressM(sqlite3_lib, "sqlite3_close")
    sqlite3_errmsg            = GetProcAddressM(sqlite3_lib, "sqlite3_errmsg")
    sqlite3_get_table         = GetProcAddressM(sqlite3_lib, "sqlite3_get_table")
    sqlite3_free_table        = GetProcAddressM(sqlite3_lib, "sqlite3_free_table")
    sqlite3_changes           = GetProcAddressM(sqlite3_lib, "sqlite3_sqlite3_changes")
    sqlite3_last_insert_rowid = GetProcAddressM(sqlite3_lib, "sqlite3_last_insert_rowid")
    sqlite3_free              = GetProcAddressM(sqlite3_lib, "sqlite3_free")
    If CallCFunctionFast(sqlite3_open, program\database, @program\dbhandle) = #SQLITE3_OK
      program\query = "CREATE TABLE appointments("
      program\query + "filename TEXT,tarrived TEXT,darrived TEXT,time TEXT,date TEXT,category TEXT,status TEXT,note TEXT,"
      program\query + "record INTEGER PRIMARY KEY AUTOINCREMENT)"
      If CallCFunctionFast(sqlite3_exec, program\dbhandle, program\query, #Null, #Null, @ReturnValue)  = #SQLITE3_OK
      EndIf
    Else
      MessageRequester("SQLite3 Error", "Could not open " + program\database)
      End
    EndIf
  Else
    MessageRequester("SQLite3 Error", "Could not initialise SQL3 Dll library")
    End
  EndIf
 
  program\query = ""

;============================================================================================================================
; Object, form colouring and resizing
;============================================================================================================================

Procedure WindowCallback(WindowID, Message, wParam, lParam)
  ReturnValue = #PB_ProcessPureBasicEvents
  If Message = #WM_CTLCOLORSTATIC Or Message = #WM_CTLCOLOREDIT Or Message = #WM_CTLCOLORLISTBOX
    Select lParam
      Case GadgetID(#Gadget_appointment_lmonitor)
        SetBkMode_(wParam, #TRANSPARENT)
        SetTextColor_(wParam, $000000)
        ReturnValue = form\yellow
      Case GadgetID(#Gadget_appointment_monitor)
        SetBkMode_(wParam, #TRANSPARENT)
        SetTextColor_(wParam, $000000)
        ReturnValue = form\yellow
    EndSelect
  EndIf
  ProcedureReturn ReturnValue
EndProcedure
 
;============================================================================================================================
; Open the main application window if you can
;============================================================================================================================

If Window_appointment()

  AddKeyboardShortcut(#Window_appointment, #PB_Shortcut_Alt    | #PB_Shortcut_X, #Shortcut_appointment_exit)  ; Program exit
  AddKeyboardShortcut(#Window_appointment, #PB_Shortcut_Return,                  #Shortcut_appointment_enter) ; Start searching

  ;------------------------------------------------------------------------------------------------
  ; Add custom balloon tooltips to all items on the screen
  ;------------------------------------------------------------------------------------------------

  MyBalloonTips(#Window_appointment, #Gadget_appointment_appointments, "Single left click on a cell to change its' contents. Double right click on a line to delete the entire line")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_badd,         "Add a new appointment to the list and the database")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bdelete,      "Delete a highlighted appointment from the list and the database")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bimport,      "Import a list of appointments into the database and the list")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bexport,      "Export the list of appointments from the database to a file on disk")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_btagall,      "Tag all the appointments in the list (Works on the corresponding database entries too)")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_buntagall,    "Untag all the appointments in the list (Works on the corresponding database entries too)")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bprint,       "Print all or selected appointments in the list (Works on the corresponding database entries too)")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bconfig,      "Configure program options such as colours, directory to monitor and others")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bhelp,        "Give program help to clueless user")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_ccategory,    "Select the category of appointments that you wish to display in the list")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_ssearch,      "Type in the text that you are searching for in here and press the ENTER key to start searching")
  MyBalloonTips(#Window_appointment, #Gadget_appointment_bexit,        "Exit the appointment book now")
 
  ;------------------------------------------------------------------------------------------------
  ; Create a new image list for the ListIconGadget and add the new icons to it.
  ;------------------------------------------------------------------------------------------------
 
  form\himagelist   = ImageList_Create_(16, 16, #ILC_MASK | #ILC_COLOR32, 0, 15)
  form\timeicon     = ImageList_ReplaceIcon_(form\himagelist, -1, UseImage(#Image_appointment_time))
  form\dateicon     = ImageList_ReplaceIcon_(form\himagelist, -1, UseImage(#Image_appointment_date))
  form\categoryicon = ImageList_ReplaceIcon_(form\himagelist, -1, UseImage(#Image_appointment_category))
  form\statusicon   = ImageList_ReplaceIcon_(form\himagelist, -1, UseImage(#Image_appointment_status))
  form\notesicon    = ImageList_ReplaceIcon_(form\himagelist, -1, UseImage(#Image_appointment_notes))
 
  SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_SETIMAGELIST, #LVSIL_SMALL, form\himagelist)
  ImageList_SetBkColor_(form\himagelist, #CLR_NONE) 
  SendMessage_(GadgetID(#Gadget_appointment_appointments), #LVM_SETEXTENDEDLISTVIEWSTYLE, #LVS_EX_SUBITEMIMAGES, #LVS_EX_SUBITEMIMAGES)
 
  ;------------------------------------------------------------------------------------------------
  ; Add the message and record statusbar icons to the statusbar now
  ;------------------------------------------------------------------------------------------------
 
  StatusBarIcon(#StatusBar_appointment, #StatusBar_appointment_messages, UseImage(#Image_appointment_messages))
  StatusBarIcon(#StatusBar_appointment, #StatusBar_appointment_records,  UseImage(#Image_appointment_records))
 
  ;------------------------------------------------------------------------------------------------
  ; Subclass the ListIconGadget and hide a stringgadget on the form to be used as a cell popup child
  ;------------------------------------------------------------------------------------------------
 
  CreateGadgetList(GadgetID(#Gadget_appointment_appointments))      ; Create the new gadget list on my ListIconGadget
  StringGadget(#Gadget_appointment_stringbox, 0, 0, 0, 0,"")        ; Create a hidden combobox on this gadget
  SendMessage_(GadgetID(#Gadget_appointment_stringbox), #EM_SETMARGINS, #EC_LEFTMARGIN | #EC_RIGHTMARGIN, 3 << 16 + 3)
  UseGadgetList(WindowID(#Window_appointment))                      ; Go back to referring to the main window gadget list
  HideGadget(#Gadget_appointment_stringbox, 1)                      ; Hide the combo box until we need it
  form\oldlistproc = SetWindowLong_(GadgetID(#Gadget_appointment_appointments), #GWL_WNDPROC, @ListIconCallBack()) ; Subclass the ListIconGadget
 
  ;------------------------------------------------------------------------------------------------
  ; Standard windows callback for form resizing, object colouring and more, produced by visual designer
  ;------------------------------------------------------------------------------------------------
 
  SetWindowCallback(@WindowCallback())
 
  ;------------------------------------------------------------------------------------------------
  ; Get windows handle for tray hiding and other functions
  ;------------------------------------------------------------------------------------------------
 
  form\winhandle = WindowID(#Window_appointment)
 
  ;------------------------------------------------------------------------------------------------
  ; Setup tray hiding For the application. These lines hide the app the moment the window is drawn
  ;------------------------------------------------------------------------------------------------
 
  ; HideWindow(#Window_appointment, 1)                                                        ; Hide the window
  ; AddSysTrayIcon(#Systray_appointment_tray, WindowID(), UseImage(#Image_appointment_tray))  ; Add a tray icon for the hidden window
  ; SysTrayIconToolTip(#Systray_appointment_tray, "SQLite Address Book")                      ; Put in a tool tip for the icon
 
  ;------------------------------------------------------------------------------------------------
  ; Setup the status bar for colouring with an API6
  ;------------------------------------------------------------------------------------------------
 
  GetAllChildHandles(form\winhandle)    ; Find status bar handle and save it to Form\Statushandle
  SendMessage_(form\statushandle, #SB_SETBKCOLOR, 0, form\colour)    ; Change the status bar colour to whatever we want now that we have found the handle to it
 
  ;------------------------------------------------------------------------------------------------
  ; Set the program day and date in the title bar
  ;------------------------------------------------------------------------------------------------
 
  SetDate(#Window_appointment)                                  ; Set current date on form title
 
  ;------------------------------------------------------------------------------------------------
  ; Get current date from pb for appointment status check
  ;------------------------------------------------------------------------------------------------
 
  program\curdate = ParseDate("%dd/%mm/%yyyy", FormatDate("%dd/%mm/%yyyy", Date()))
 
  ;------------------------------------------------------------------------------------------------
  ; See if there is an INI file present, get monitor path and create one if not
  ;------------------------------------------------------------------------------------------------
 
  If FileSize(program\inifile) =  -1
    MessageRequester("Program Error","Configuration file not found, create a new one now please", #PB_MessageRequester_Ok)
    GetMonitorPath()                                       ; Get new path and write to ini file
  Else
    ;----------------------------------------------------------------------------------------------
    ; Get program startup values from the INI file because it exists here
    ;----------------------------------------------------------------------------------------------
    program\aptdir = GetIniVal("Monitor Dir", "Directory To Monitor")  ; Get the directory that you will be monitoring
    SetGadgetText(#Gadget_appointment_monitor, program\aptdir)
    ;----------------------------------------------------------------------------------------------
    ; Create the thread that monitors the directory now
    ;----------------------------------------------------------------------------------------------
    program\dirhandle = CreateFile_(program\aptdir, #FILE_LIST_DIRECTORY, #FILE_SHARE_WRITE | #FILE_SHARE_READ | #FILE_SHARE_DELETE, #Null, #OPEN_EXISTING, #FILE_FLAG_BACKUP_SEMANTICS, #Null)
    program\threadid  = CreateThread(@RDCW(), program\dirhandle)
  EndIf
 
  ;------------------------------------------------------------------------------------------------
  ; Do certain things at first program run
  ;------------------------------------------------------------------------------------------------
  program\firstrun = 1

  ;------------------------------------------------------------------------------------------------
  ; Set intial program quit value to 0 in case it has a trash value at startup
  ;------------------------------------------------------------------------------------------------
  program\progquit = 0

  ;------------------------------------------------------------------------------------------------
  ; Fill the category box in the main form with unique categories
  ;------------------------------------------------------------------------------------------------
  FillCombo(#Gadget_appointment_ccategory,   "Select DISTINCT category From appointments ORDER BY category")

  ;------------------------------------------------------------------------------------------------
  ; Count the number of records in the database and tell the user via the status bar
  ;------------------------------------------------------------------------------------------------
  program\numitems = CountRecords()
 
  If program\numitems = 0
    WriteError("Main Form")
  EndIf
 
  ;------------------------------------------------------------------------------------------------
  ; Show all the loaded data in the display list ad update status on the fly as you do
  ;------------------------------------------------------------------------------------------------
  ShowData()
 
  ;------------------------------------------------------------------------------------------------
  ; Let the user know that the database is open via the status bar
  ;------------------------------------------------------------------------------------------------
  SetStat(#StatusBar_appointment_messages, "Information: Database " + program\database + " open")

  ;------------------------------------------------------------------------------------------------
  ; Set focus to the main program window now
  ;------------------------------------------------------------------------------------------------
 
  ActivateWindow()

  ;------------------------------------------------------------------------------------------------
  ; Activate form and set focus to category box
  ;------------------------------------------------------------------------------------------------
  SetGadgetState(#Gadget_appointment_ccategory, 0)

  ;------------------------------------------------------------------------------------------------
  ; Now check all window events
  ;------------------------------------------------------------------------------------------------

  Repeat                                          ; Continually loop through all menuu events

    ;----------------------------------------------------------------------------------------------
    ; Check to see if the application has been hidden and show icon in tray for it
    ;----------------------------------------------------------------------------------------------

    If IsIconic_(form\winhandle) <> 0              ; The window has been minimized, so hide to tray
      HideWindow(#Window_appointment, 1)
      AddSysTrayIcon(#Systray_appointment_tray, form\winhandle, UseImage(#Image_appointment_tray))
      SysTrayIconToolTip(#Systray_appointment_tray, "Appointment Reader, " + #Version)
    EndIf

    ;----------------------------------------------------------------------------------------------
    ; What sort of event happened on our window
    ;----------------------------------------------------------------------------------------------

    Select WaitWindowEvent()

      ;--------------------------------------------------------------------------------------------
      ; Someone clicked on a form, resized the window etc
      ;--------------------------------------------------------------------------------------------

      Case #PB_Event_CloseWindow
        Select EventWindowID()
          Case #Window_appointment                  : program\progquit = 1
        EndSelect

      ;--------------------------------------------------------------------------------------------------------
      ; Someone used a menu item or keyboard shortcut on our form
      ;--------------------------------------------------------------------------------------------------------

      Case #PB_Event_Menu
        Select EventMenuID()
          Case #Shortcut_appointment_exit           : program\progquit = 1
          Case #Shortcut_appointment_enter
        EndSelect

      ;--------------------------------------------------------------------------------------------------------
      ; Someone used a gadget on our form
      ;--------------------------------------------------------------------------------------------------------

      Case #PB_Event_Gadget
        Select EventGadgetID()
          Case #Gadget_appointment_appointments
            Select EventType()
              Case #PB_EventType_LeftClick          : ProcessCellSelection()      ; Process a cell edit
              Case #PB_EventType_RightClick         : DeleteRecord()              ; Delete the highlighted item
            EndSelect
          Case #Gadget_appointment_badd             : AddRecord()
          Case #Gadget_appointment_bdelete          : DeleteRecord()              ; Delete the highlighted item
          Case #Gadget_appointment_bconfig          : GetMonitorPath()            ; Get the path to monitor
          Case #Gadget_appointment_bimport
          Case #Gadget_appointment_bexport
          Case #Gadget_appointment_btagall
          Case #Gadget_appointment_buntagall
          Case #Gadget_appointment_bprint
          Case #Gadget_appointment_bhelp
          Case #Gadget_appointment_ccategory        : ShowData()
          Case #Gadget_appointment_bexit            : program\progquit = 1
        EndSelect

        ;-------------------------------------------------------------------------------------------------------
        ; Someone clicked on the application's tray icon
        ;-------------------------------------------------------------------------------------------------------

        Case #PB_Event_SysTray                                              ; System tray events
         If EventType() = #PB_EventType_LeftClick                           ; If the app's icon was clicked, unhide from tray
           RemoveSysTrayIcon(#Systray_appointment_tray)                     ; Remove the icon from the tray
           HideWindow(#Window_appointment, 0)                               ; Show the application window
           SetForegroundWindow_(form\winhandle)                             ; Un-hiding the window brings it to the front
         EndIf
       
         ;------------------------------------------------------------------------------------------------------
         ; No more form events to process
         ;------------------------------------------------------------------------------------------------------

    EndSelect

  Until program\progquit                                                    ; No more tests to do

  ;----------------------------------------------------------------------------------------------------------
  ; Free monitor, thread and objects
  ;----------------------------------------------------------------------------------------------------------

  If program\dirhandle                      ; Only do this if the thread is alive and dir object has a handle
    FreeMemory(*fni)                                                        ; Free monitor memory
    KillThread(program\threadid)                                            ; Kill the monitor thread
    CloseHandle_(program\dirhandle)                                         ; Close the monitor object
  EndIf                                                                     ; End test

  ;----------------------------------------------------------------------------------------------------------
  ; Close the function library that we were using
  ;----------------------------------------------------------------------------------------------------------

  If IsLibrary(0)                                                           ; If the library was open, close it
    CloseLibrary(0)
  EndIf

  CloseWindow(#Window_appointment)                                          ; Close the program window

EndIf                                                                       ; End the conditional test

End                                                                         ; Remove program stub from memory.

;----------------------------------------------------------------------------------------------------------
; End! ! Nothing to see here. Movel along little doggies!
;----------------------------------------------------------------------------------------------------------

; Procedure.b HasLetters(myString.s) ; Xombie!
;    *Position.l
;    HoldLength.l
;    HoldLength = Len(myString)
;    *Position = @myString
;    While *Position - @myString < HoldLength
;       If PeekB(*Position) < 48 Or PeekB(*Position) > 57
;         ProcedureReturn #True
;       EndIf
;       *Position + 1
;    Wend
;    ProcedureReturn #False
; EndProcedure
; 
; Debug HasLetters("1a024")



; IDE Options = PureBasic 5.22 LTS (Windows - x86)
; Folding = ------
; EnableXP