﻿;==========================================================================================================================================================
; Run a timed background check of current appointments
;==========================================================================================================================================================

Procedure CheckAppointments(ObjectHandle.i, Message.i, EventNumber.i, TimePeriod.i)
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; Make sure no other window tries to open
    ;----------------------------------------------------------------------------------------------
    
    Program\DontCheck = 1
    
    ;----------------------------------------------------------------------------------------------
    ; Check for the correct timer event
    ;----------------------------------------------------------------------------------------------
    
    Select EventNumber.i
        
      ;--------------------------------------------------------------------------------------------
      ; User selected timer event number, to be checked every ten minutes
      ;--------------------------------------------------------------------------------------------
      
      Case #Event_Alarm_Check
      
        ;------------------------------------------------------------------------------------------
        ; Show appointments for records whose alarm is the same as the current date
        ;------------------------------------------------------------------------------------------
        
        QueryString.s = "SELECT * FROM Appointments "
        
        QueryString.s + "WHERE appAlarm = '" + IsoDate(Program\CurrentDate, "IsoDate") + "' "
        
        QueryString.s + "AND appStatus <> 'Finished appointment' "
        
        QueryString.s + "AND appDeleted <> '1' "
        
;       QueryString.s + "LIMIT 1"
        
        ;------------------------------------------------------------------------------------------
        ; Mimimise the amount of typing by aliasing structures with their name as a preface
        ;------------------------------------------------------------------------------------------
        
        With Appointment
          
          ;----------------------------------------------------------------------------------------
          ; Run the database query
          ;----------------------------------------------------------------------------------------
          
          If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)
            
            ;--------------------------------------------------------------------------------------
            ; Iterate through all rows of returned data
            ;--------------------------------------------------------------------------------------
            
            While NextDatabaseRow(Program\DatabaseHandle)
              
              ;------------------------------------------------------------------------------------
              ; Return all the program strings into structured variables
              ;------------------------------------------------------------------------------------
              
              \appName        = KillQuote(GetDatabaseString(Program\DatabaseHandle,   0))
              
              \appDate        =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  1), "NormalDate")
              
              \appTime        =            GetDatabaseString(Program\DatabaseHandle,  2) 
               
              \appAlarm       =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  3), "NormalDate")
              
              \appRepeat      =            GetDatabaseString(Program\DatabaseHandle,  4) 
              
              \appProtect     =            GetDatabaseString(Program\DatabaseHandle,  5) 
              
              \appCategory    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  6))
              
              \appStatus      =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  7))
              
              \appComment     =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  8))
              
              \appArchived    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  9))
              
              \appEmailed     =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 10))
              
              \appSound       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 11))
              
              \appDeleted     =            GetDatabaseString(Program\DatabaseHandle, 12) 
              
              \appRecord      =            GetDatabaseString(Program\DatabaseHandle, 13) 
              
              ;------------------------------------------------------------------------------------
              ; Spawn an alarm window if it isn't already open and then fill it with the contents of
              ; the above structure which is then cleared. If the window already exists, the next
              ; alarm is just added to it.
              ;------------------------------------------------------------------------------------
              
              SpawnAlarmWindow()
              
              ;------------------------------------------------------------------------------------
              ; Get the next line of returned data
              ;------------------------------------------------------------------------------------
              
            Wend
            
            ;--------------------------------------------------------------------------------------
            ; Close off and flush the database query
            ;--------------------------------------------------------------------------------------
            
            FinishDatabaseQuery(Program\DatabaseHandle)
            
            ;--------------------------------------------------------------------------------------
            ; Let the user know if an error occurred
            ;--------------------------------------------------------------------------------------
            
          Else
            
            SetInfoBarArea("Headings", "Error", #InfoStringQueryFailed + DatabaseError(), "CheckAppointments")
            
          EndIf
          
          ;----------------------------------------------------------------------------------------
          ; WIth have finished with the structure shortcut prefacing
          ;----------------------------------------------------------------------------------------
          
        EndWith
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
    EndSelect
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DontCheck = 0
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure
; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 28
; FirstLine = 41
; Folding = -
; EnableXP