﻿;==========================================================================================================================================================
; Spawn an alarm window for each alarm that needs to open
;==========================================================================================================================================================

Procedure SpawnAlarmWindow()
  
  ;------------------------------------------------------------------------------------------------
  ; If the window isn't already open, open it
  ;------------------------------------------------------------------------------------------------
  
  If IsWindow(#Window_AppointmentAlarm) = 0
    
    ;----------------------------------------------------------------------------------------------
    ; Set this window on top of others or alarm window will not show
    ;----------------------------------------------------------------------------------------------
    
    If Window_AppointmentAlarm()
      
      ;--------------------------------------------------------------------------------------------
      ; Prevent user input on the main window
      ;--------------------------------------------------------------------------------------------
      
      DisableWindow(#Window_AppointmentMuncher, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Set this window on top of others or alarm window will not show
      ;--------------------------------------------------------------------------------------------
      
      StickyWindow(#Window_AppointmentAlarm, #True)
      
      ;--------------------------------------------------------------------------------------------
      ; Set this window's title
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_AppointmentAlarm, Program\AlarmWindowTitle)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the various keyboard shortcuts
      ;--------------------------------------------------------------------------------------------
      
      AddKeyboardShortcut(#Window_AppointmentAlarm, #PB_Shortcut_Escape, #Shortcut_AppointmentAlarm_Escape)
      
      ;--------------------------------------------------------------------------------------------
      ; Add the chosen icons to the main list
      ;--------------------------------------------------------------------------------------------
      
      BuildImageList(#Gadget_AppointmentAlarm_Appointments)
      
      ;--------------------------------------------------------------------------------------------
      ; Hover enabled image gadgets posing as buttons
      ;--------------------------------------------------------------------------------------------
      
      AddHoverGadget(GadgetID(#Gadget_AppointmentAlarm_Snooze))
      
      AddHoverGadget(GadgetID(#Gadget_AppointmentAlarm_StopSound))
      
      AddHoverGadget(GadgetID(#Gadget_AppointmentAlarm_Cancel))
      
      ;--------------------------------------------------------------------------------------------
      ; Remove theme support from the comment box or it retains a border even when API removed
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTheme_(GadgetID(#Gadget_AppointmentAlarm_Comment),   @null.w, @null.w)
      
      ;--------------------------------------------------------------------------------------------
      ; Play the default alarm sound
      ;--------------------------------------------------------------------------------------------
      
      If Setup\AlarmSound <> 0
        
        Select  Setup\AlarmSound
            
          Case 1  : PlaySound(#SndStandard, #PB_Sound_MultiChannel | #PB_Sound_Loop, 100)  : Delay(1000)
            
        EndSelect
        
      EndIf
        
      ;--------------------------------------------------------------------------------------------
      ; Cannot open the appointment alarm window, usually a system error
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Error",  #InfoStringCannotOpen, "SpawnAlarmWindow")
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; The alarm window is already open.
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Info",  #InfoStringWindowOpen)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the passed appointment already exists in the list and set a flag if found
  ;------------------------------------------------------------------------------------------------
  
  NumberOfItems.i = CountGadgetItems(#Gadget_AppointmentAlarm_Appointments)
  
  For CheckExisting.i = 0 To NumberOfItems.i - 1
    
    ListRecordNumber.s = GetGadgetItemText(#Gadget_AppointmentAlarm_Appointments, CheckExisting.i, Program\AlarmRecordColumn)
    
    If ListRecordNumber.s = Appointment\appRecord
      
      FoundFlag.i = 1
      
    EndIf
    
  Next CheckExisting.i
  
  ;------------------------------------------------------------------------------------------------
  ; Add the passed appointment to the list if it does not already exist there
  ;------------------------------------------------------------------------------------------------
  
  If FoundFlag.i = 0
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
    With Appointment
      
      ;--------------------------------------------------------------------------------------------
      ;
      ;--------------------------------------------------------------------------------------------
      
      AddGadgetItem(#Gadget_AppointmentAlarm_Appointments, -1, \appName + Chr(10) + \appDate + Chr(10) + \appTime + Chr(10) + \appComment + Chr(10) + \appRecord)
      
      ;--------------------------------------------------------------------------------------------
      ; Select the correct category icon into the first column
      ;--------------------------------------------------------------------------------------------
      
      Select Appointment\appCategory
          
        Case  "Unknown"           : CategoryIcon.i  = Icon\CatUnknown
          
        Case  "Aeroplane"         : CategoryIcon.i  = Icon\CatAeroplane
          
        Case  "Amateur"           : CategoryIcon.i  = Icon\CatAmateur
          
        Case  "Ambulance"         : CategoryIcon.i  = Icon\CatAmbulance
          
        Case  "Anyu"              : CategoryIcon.i  = Icon\CatAnyu
          
        Case  "Bank"              : CategoryIcon.i  = Icon\CatBank
          
        Case  "Bus"               : CategoryIcon.i  = Icon\CatBus
          
        Case  "Business"          : CategoryIcon.i  = Icon\CatBusiness
          
        Case  "Camel"             : CategoryIcon.i  = Icon\CatCamel
          
        Case  "Cbradio"           : CategoryIcon.i  = Icon\CatCbradio
          
        Case  "Cfa"               : CategoryIcon.i  = Icon\CatCfa
          
        Case  "Charity"           : CategoryIcon.i  = Icon\CatCharity
          
        Case  "Dentist"           : CategoryIcon.i  = Icon\CatDentist
          
        Case  "Doctor"            : CategoryIcon.i  = Icon\CatDoctor
          
        Case  "Fire"              : CategoryIcon.i  = Icon\CatFire
          
        Case  "Fireman"           : CategoryIcon.i  = Icon\CatFireman
          
        Case  "Hospital"          : CategoryIcon.i  = Icon\CatHospital
          
        Case  "Medical"           : CategoryIcon.i  = Icon\CatMedical
          
        Case  "Optometrist"       : CategoryIcon.i  = Icon\CatOptometrist
          
        Case  "Personal"          : CategoryIcon.i  = Icon\CatPersonal
          
        Case  "Physiotherapist"   : CategoryIcon.i  = Icon\CatPhysiotherapist
          
        Case  "Police"            : CategoryIcon.i  = Icon\CatPolice
          
        Case  "Scouts"            : CategoryIcon.i  = Icon\CatScouts
          
        Case  "Service"           : CategoryIcon.i  = Icon\CatService
          
        Case  "Shopping"          : CategoryIcon.i  = Icon\CatShopping
          
        Case  "Taxi"              : CategoryIcon.i  = Icon\CatTaxi
          
        Case  "Tradesman"         : CategoryIcon.i  = Icon\CatTradesman
          
        Case  "Train"             : CategoryIcon.i  = Icon\CatTrain
          
        Case  "Truck"             : CategoryIcon.i  = Icon\CatTruck
          
        Case  "Veterinarian"      : CategoryIcon.i  = Icon\CatVeterinarian
          
      EndSelect
      
      ChangeIcon(#Gadget_AppointmentAlarm_Appointments, NumberOfItems.i, 0, CategoryIcon.i)
        
      ;------------------------------------------------------------------------------------------------
      ; Show the active icon as only active appointments are passed through this far
      ;------------------------------------------------------------------------------------------------
      
      ChangeIcon(#Gadget_AppointmentAlarm_Appointments, NumberOfItems.i, 1, Icon\StatusActive)
          
      ;------------------------------------------------------------------------------------------------
      ; Select the correct alarm time of day icon.   yrreti code
      ;------------------------------------------------------------------------------------------------
      
      HourCheck.i  = Val(ReplaceString(Appointment\appTime, ":", "", 1))
      
      Select HourCheck.i
          
        Case    0 To 1159, 2400 : ChangeIcon(#Gadget_AppointmentAlarm_Appointments, NumberOfItems.i, Program\AlarmTimeColumn, Icon\LightMorning)
          
          ;Debug Appointment\appName + " Time check [" + Str(HourCheck.i)  + "] 0 To 1159, 2400"
          
        Case 1200 To 1759       : ChangeIcon(#Gadget_AppointmentAlarm_Appointments, NumberOfItems.i, Program\AlarmTimeColumn, Icon\LightNoon)
          
          ;Debug Appointment\appName + " Time check [" + Str(HourCheck.i)  + "] 1200 To 1759"
          
        Case 1800 To 2359       : ChangeIcon(#Gadget_AppointmentAlarm_Appointments, NumberOfItems.i, Program\AlarmTimeColumn, Icon\LightNight)
          
          ;Debug Appointment\appName + " Time check [" + Str(HourCheck.i)  + "] 1800 To 2359"
          
      EndSelect
      
      ;--------------------------------------------------------------------------------------------
      ;
      ;--------------------------------------------------------------------------------------------
      
      NumberOfItems.i = CountGadgetItems(#Gadget_AppointmentAlarm_Appointments)
      
      ;--------------------------------------------------------------------------------------------
      ;
      ;--------------------------------------------------------------------------------------------
      
      SetWindowTitle(#Window_AppointmentAlarm, Program\AlarmWindowTitle + ", " + Str(NumberOfItems.i))
      
      ;--------------------------------------------------------------------------------------------
      ;
      ;--------------------------------------------------------------------------------------------
      
    EndWith
    
    ;----------------------------------------------------------------------------------------------
    ;
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Info",  #InfoStringAlarmExists)
    
  EndIf
  
  
  ;------------------------------------------------------------------------------------------------
  ; Clear the original structure to prevent confusion
  ;------------------------------------------------------------------------------------------------
  
  ClearStructure(@Appointment, AppointmentData)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure

; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 54
; FirstLine = 52
; Folding = -
; EnableXP