﻿;==========================================================================================================================================================
; Show a list of all the keyboard shortcuts
;==========================================================================================================================================================

Procedure KeysAppointments()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; Check if a previous window is open
    ;----------------------------------------------------------------------------------------------
    
    If Program\WindowMutex <> 1
      
      ;--------------------------------------------------------------------------------------------
      ; Try to open the data window
      ;--------------------------------------------------------------------------------------------
      
      If Window_AppointmentKeys()
        
        ;------------------------------------------------------------------------------------------
        ; Prevent user input on the main window
        ;------------------------------------------------------------------------------------------
        
        DisableWindow(#Window_AppointmentMuncher, #True)
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components a window is open and not to attempt to open another
        ;------------------------------------------------------------------------------------------
        
        Program\WindowMutex = 1
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components that the alarm checker should not be run right now
        ;------------------------------------------------------------------------------------------
        
        Program\DontCheck = 1
        
        ;------------------------------------------------------------------------------------------
        ; Add the various keyboard shortcuts
        ;------------------------------------------------------------------------------------------
        
        AddKeyboardShortcut(#Window_AppointmentKeys, #PB_Shortcut_Escape, #Shortcut_AppointmentKeys_Escape)
        
        ;------------------------------------------------------------------------------------------
        ; Set the common window and statusbar message
        ;------------------------------------------------------------------------------------------
        
        Message.s = "Show keyboard shortcuts"
        
        ;------------------------------------------------------------------------------------------
        ; Use the common message as a window identifier
        ;------------------------------------------------------------------------------------------
        
        Program\LastWindow = Message.s
        
        ;------------------------------------------------------------------------------------------
        ; Set the window title with the common message and next record number
        ;------------------------------------------------------------------------------------------
        
        SetWindowTitle(#Window_AppointmentKeys, Message.s)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        Keys.s  = "Keyboard Shortcut: "
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutAddAppointment,       Keys.s, "", 1)  + Chr(10) + #BalloonTextAddAppointment,       ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutCopyAppointment,      Keys.s, "", 1)  + Chr(10) + #BalloonTextCopyAppointment,      ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutDeleteAppointment,    Keys.s, "", 1)  + Chr(10) + #BalloonTextDeleteAppointment,    ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutEditAppointment,      Keys.s, "", 1)  + Chr(10) + #BalloonTextEditAppointment,      ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutFindAppointment,      Keys.s, "", 1)  + Chr(10) + #BalloonTextFindAppointment,      ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutHelpAppointment,      Keys.s, "", 1)  + Chr(10) + #BalloonTextHelpAppointment,      ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutPrintAppointment,     Keys.s, "", 1)  + Chr(10) + #BalloonTextPrintAppointment,     ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutUndeleteAppointment,  Keys.s, "", 1)  + Chr(10) + #BalloonTextUndeleteAppointment,  ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutExitAppointment,      Keys.s, "", 1)  + Chr(10) + #BalloonTextExitAppointment,      ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutAboutAppointments,    Keys.s, "", 1)  + Chr(10) + #BalloonTextAboutAppointments,    ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutKeyboardShortcuts ,   Keys.s, "", 1)  + Chr(10) + #BalloonTextKeyboardShortcuts,    ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutToggleTooltips,       Keys.s, "", 1)  + Chr(10) + #BalloonTextToggleTooltips,       ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutToggleStickywindow,   Keys.s, "", 1)  + Chr(10) + #BalloonTextToggleStickywindow,   ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutToggleTitleClock,     Keys.s, "", 1)  + Chr(10) + #BalloonTextToggleTitleClock,     ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutToggleScreenSnap,     Keys.s, "", 1)  + Chr(10) + #BalloonTextToggleScreenSnap,     ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutCleandatabase,        Keys.s, "", 1)  + Chr(10) + #BalloonTextCleanDatabase,        ImageID(#Shortcuts))
        
        AddGadgetItem(#Gadget_AppointmentKeys_Keylist, -1, ReplaceString(#ShortcutSetupprogram,         Keys.s, "", 1)  + Chr(10) + #BalloonTextSetupprogram,         ImageID(#Shortcuts))
        
        ;------------------------------------------------------------------------------------------
        ; Set the current operation details on the the main window status bar
        ;------------------------------------------------------------------------------------------
        
        SetInfoBarArea("Headings",  "Info",  Message.s)
        
        ;------------------------------------------------------------------------------------------
        ; Set focus on the appointment name field
        ;------------------------------------------------------------------------------------------
        
        SetActiveGadget(#Gadget_AppointmentKeys_Keylist)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      Else
        
        SetInfoBarArea("Headings",  "Error",  #InfoStringCannotOpen)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Warn",  #InfoStringWindowOpen)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure



; IDE Options = PureBasic 5.22 LTS (Windows - x86)
; CursorPosition = 94
; FirstLine = 64
; Folding = -
; EnableXP