﻿;==========================================================================================================================================================
; Setup various program options and switches
;==========================================================================================================================================================

Procedure SetupAppointments()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; Check if a previous window is open
    ;----------------------------------------------------------------------------------------------
    
    If Program\WindowMutex <> 1
      
      ;--------------------------------------------------------------------------------------------
      ; Try to open the data window
      ;--------------------------------------------------------------------------------------------
      
      If Window_AppointmentSetup()
        
        ;------------------------------------------------------------------------------------------
        ; Prevent user input on the main window
        ;------------------------------------------------------------------------------------------
        
        DisableWindow(#Window_AppointmentMuncher, #True)
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components a window is open and not to attempt to open another
        ;------------------------------------------------------------------------------------------
        
        Program\WindowMutex = 1
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components that the alarm checker should not be run right now
        ;------------------------------------------------------------------------------------------
        
        Program\DontCheck = 1
        
        ;------------------------------------------------------------------------------------------
        ; Add the various keyboard shortcuts
        ;------------------------------------------------------------------------------------------
        
        AddKeyboardShortcut(#Window_AppointmentSetup, #PB_Shortcut_Escape, #Shortcut_AppointmentSetup_Escape)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        AddHoverGadget(GadgetID(#Gadget_AppointmentSetup_Save))
        
        AddHoverGadget(GadgetID(#Gadget_AppointmentSetup_Exit))
        
        ;------------------------------------------------------------------------------------------
        ; Add the possible alarm snooze settings to the main form
        ;------------------------------------------------------------------------------------------
        
        Restore BeginSnoozeSettings
        
        While TempCategory.s <> "_EndSnoozeSettings_"
          
          Read.s TempCategory.s
          
          If TempCategory.s <> "_EndSnoozeSettings_"
            
            AddGadgetItem(#Gadget_AppointmentSetup_SnoozePeriod, -1, TempCategory.s, ImageID(#SnoozeClock))
            
          EndIf
          
        Wend
        
        TempCategory.s    = "<>"
        
        ;------------------------------------------------------------------------------------------
        ; Add the possible alarm snooze settings to the main form
        ;------------------------------------------------------------------------------------------
        
        Restore StartupDates
        
        While TempCategory.s <> "_EndStartupDates_"
          
          Read.s TempCategory.s
          
          If TempCategory.s <> "_EndStartupDates_"
            
            AddGadgetItem(#Gadget_AppointmentSetup_StartupDates, -1, TempCategory.s, ImageID(#Clipboard))
            
          EndIf
          
        Wend
        
        TempCategory.s    = "<>"
        
        ;------------------------------------------------------------------------------------------
        ; Add the possible alarm snooze settings to the main form
        ;------------------------------------------------------------------------------------------
        
        Restore AlarmSounds
        
        While TempCategory.s <> "_EndAlarmSounds_"
          
          Read.s TempCategory.s
          
          If TempCategory.s <> "_EndAlarmSounds_"
            
            AddGadgetItem(#Gadget_AppointmentSetup_AlarmSound, -1, TempCategory.s, ImageID(#Alarmsound))
            
          EndIf
          
        Wend
        
        TempCategory.s    = "<>"
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        SetGadgetState(#Gadget_AppointmentSetup_Titleclock,     Setup\TitleClock)
        
        SetGadgetState(#Gadget_AppointmentSetup_Stickywindow,   Setup\StickyWindow)
        
        SetGadgetState(#Gadget_AppointmentSetup_Tooltips,       Setup\ToolTips)
        
        SetGadgetState(#Gadget_AppointmentSetup_Screensnap,     Setup\Screensnap)
        
        SetGadgetState(#Gadget_AppointmentSetup_AutoArchive,    Setup\AutoArchive)
        
        SetGadgetState(#Gadget_AppointmentSetup_EmailAlarm,     Setup\EmailAlarm)
      
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        SetGadgetState(#Gadget_AppointmentSetup_StartupDates,   Setup\StartupDates)
        
        SetGadgetState(#Gadget_AppointmentSetup_SnoozePeriod,   Setup\SnoozePeriod)
        
        SetGadgetState(#Gadget_AppointmentSetup_AlarmSound,     Setup\AlarmSound)
        
        SetGadgetText(#Gadget_AppointmentSetup_LastSearch,      Setup\LastSearch)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        SetGadgetText(#Gadget_AppointmentSetup_ServerName,      Setup\ServerName)
  
        SetGadgetText(#Gadget_AppointmentSetup_PortNumber,      Setup\PortNumber)
  
        SetGadgetText(#Gadget_AppointmentSetup_UserName,        Setup\UserName)
  
        SetGadgetText(#Gadget_AppointmentSetup_Password,        Setup\Password)
  
        SetGadgetText(#Gadget_AppointmentSetup_NeedSsl,         Setup\NeedSsl)
  
        SetGadgetText(#Gadget_AppointmentSetup_EmailAddress,    Setup\EmailAddress)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      Else
        
        SetInfoBarArea("Headings",  "Error",  #InfoStringCannotOpen)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Warn",  #InfoStringWindowOpen)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 133
; FirstLine = 159
; Folding = -
; EnableXP