﻿;==========================================================================================================================================================
; Add a new appointment to the list
;==========================================================================================================================================================

Procedure AddNewAppointment()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; Check if a previous window is open
    ;----------------------------------------------------------------------------------------------
    
    If Program\WindowMutex <> 1
      
      ;--------------------------------------------------------------------------------------------
      ; Try to open the data window
      ;--------------------------------------------------------------------------------------------
      
      If Window_AppointmentData()
        
        ;------------------------------------------------------------------------------------------
        ; Prevent user input on the main window
        ;------------------------------------------------------------------------------------------
        
        DisableWindow(#Window_AppointmentMuncher, #True)
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components a window is open and not to attempt to open another
        ;------------------------------------------------------------------------------------------
        
        Program\WindowMutex = 1
        
        ;------------------------------------------------------------------------------------------
        ; Warn other components that the alarm checker should not be run right now
        ;------------------------------------------------------------------------------------------
        
        Program\DontCheck = 1
        
        ;------------------------------------------------------------------------------------------
        ; Add the various keyboard shortcuts
        ;------------------------------------------------------------------------------------------
        
        AddKeyboardShortcut(#Window_AppointmentData, #PB_Shortcut_Escape, #Shortcut_AppointmentData_Escape)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        AddHoverGadget(GadgetID(#Gadget_AppointmentData_Save))
      
        AddHoverGadget(GadgetID(#Gadget_AppointmentData_Exit))
        
        ;------------------------------------------------------------------------------------------
        ;  1  Set the name of appointment field if used
        ;------------------------------------------------------------------------------------------
        

        ;------------------------------------------------------------------------------------------
        ;  2  Set the date fields to the current date and disable the checkboxes until the user ticks them
        ;------------------------------------------------------------------------------------------
        
        SetGadgetText(#Gadget_AppointmentData_Itemdate, Program\CurrentDate)

        ;------------------------------------------------------------------------------------------
        ;  3  Time
        ;------------------------------------------------------------------------------------------
        
        SetGadgetState(#Gadget_AppointmentData_Timepick,  0)

        ;------------------------------------------------------------------------------------------
        ;  4  Alarm
        ;------------------------------------------------------------------------------------------
        
        SetGadgetState(#Gadget_AppointmentData_Alarmpick, 0)
        
        ;------------------------------------------------------------------------------------------
        ;  5  Repeat alarm choices
        ;------------------------------------------------------------------------------------------
        
        Restore AlarmRepeats
        
        While TempCategory.s <> "_EndAlarmRepeats_"
          
          Read.s TempCategory.s
          
          If TempCategory.s <> "_EndAlarmRepeats_"
            
            AddGadgetItem(#Gadget_AppointmentData_Repeat, -1, TempCategory.s, ImageID(#Repeatalarm))
            
          EndIf
          
        Wend
        
        TempCategory.s    = "<>"
        
        TempIconNumber.s  = "<>"
        
        SetGadgetState(#Gadget_AppointmentData_Repeat, 0)
        
        ;------------------------------------------------------------------------------------------
        ;  6  Protect record choices
        ;------------------------------------------------------------------------------------------
        
        AddGadgetItem(#Gadget_AppointmentData_Protected, -1, "Unlocked",  ImageID(#Unlocked))
        
        AddGadgetItem(#Gadget_AppointmentData_Protected, -1, "Locked",    ImageID(#Locked))
        
        SetGadgetState(#Gadget_AppointmentData_Protected, 0)

        ;------------------------------------------------------------------------------------------
        ;  7  Setup the category items
        ;------------------------------------------------------------------------------------------
        
        Restore BeginCategories
        
        CategoryStartValue.i = #CatUnknown
        
        While TempCategory.s <> "_EndCategories_"
          
          Read.s TempCategory.s
          
          Read.s TempIconNumber.s
          
          If TempCategory.s <> "_EndCategories_"
            
            AddGadgetItem(#Gadget_AppointmentData_Category, -1, TempCategory.s, ImageID(CategoryStartValue.i))
            
            CategoryStartValue.i + 1
            
          EndIf
          
        Wend
        
        TempCategory.s    = "<>"
        
        TempIconNumber.s  = "<>"
        
        SetGadgetState(#Gadget_AppointmentData_Category, 0)
        
        ;------------------------------------------------------------------------------------------
        ;  8  Add the possible status conditions to the status chooser
        ;------------------------------------------------------------------------------------------
        
        AddGadgetItem(#Gadget_AppointmentData_Itemstatus, -1, "Active appointment",   ImageID(#StatusActive))
        
        AddGadgetItem(#Gadget_AppointmentData_Itemstatus, -1, "Finished appointment", ImageID(#StatusFinished))

        ;------------------------------------------------------------------------------------------
        ;  9  The alarm sounds selection
        ;------------------------------------------------------------------------------------------
        
;         Restore AlarmSounds
;         While TempCategory.s <> "_EndAlarmSounds_"
;           Read.s TempCategory.s
;           If TempCategory.s <> "_EndAlarmSounds_"
;             AddGadgetItem(#Gadget_AppointmentData_AlarmSound, -1, TempCategory.s, ImageID(#Alarmsound))
;           EndIf
;         Wend
;         TempCategory.s    = "<>"
;         TempIconNumber.s  = "<>"
;         SetGadgetState(#Gadget_AppointmentData_AlarmSound, 1)
        
        ;------------------------------------------------------------------------------------------
        ; 10  Remove theme support from the comment box or it retains a border even when API removed
        ;------------------------------------------------------------------------------------------
        
        SetWindowTheme_(GadgetID(#Gadget_AppointmentData_Comment),   @null.w, @null.w)

        ;------------------------------------------------------------------------------------------
        ; 11  Get the last record number in the database and give the next probable record number
        ;------------------------------------------------------------------------------------------

        NextRecord.i = Val(LastRecord("Appointments")) + 1
        
        ;------------------------------------------------------------------------------------------
        ; 12  Set the record number field with the probable next record number
        ;------------------------------------------------------------------------------------------
  
        SetGadgetText(#Gadget_AppointmentData_Recordnumber, Str(NextRecord.i) + " ?")
        
        ;------------------------------------------------------------------------------------------
        ; 13 Set the common window and statusbar message
        ;------------------------------------------------------------------------------------------
        
        Message.s = "Add New Appointment"
      
        ;------------------------------------------------------------------------------------------
        ; 14  Use the common message as a window identifier
        ;------------------------------------------------------------------------------------------
        
        Program\LastWindow = Message.s
        
        ;------------------------------------------------------------------------------------------
        ; 15  Set the status field to "Active appointment" or "Finished appointment"
        ;------------------------------------------------------------------------------------------
        
        SetGadgetText(#Gadget_AppointmentData_Itemstatus,   "Active appointment")
        
        ;------------------------------------------------------------------------------------------
        ; 16  Set the window title with the common message and next record number
        ;------------------------------------------------------------------------------------------
        
        SetWindowTitle(#Window_AppointmentData, Message.s + "   --   Record number: " + Str(NextRecord.i) + " ?")
        
        ;------------------------------------------------------------------------------------------
        ; 17  Set the current operation details on the the main window status bar
        ;------------------------------------------------------------------------------------------
        
        SetInfoBarArea("Headings",  "Info",  Message.s + "   --   Record number: " + Str(NextRecord.i) + " ?")
        
        ;------------------------------------------------------------------------------------------
        ; 18  Set focus on the appointment name field
        ;------------------------------------------------------------------------------------------
        
        SetActiveGadget(#Gadget_AppointmentData_Itemname)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
      
      Else
        
        SetInfoBarArea("Headings",  "Error",  #InfoStringCannotOpen, "AddNewAppointment")
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Warn",  #InfoStringWindowOpen)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
      
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
      
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 145
; FirstLine = 193
; Folding = -
; EnableXP