﻿;==========================================================================================================================================================
; Display the current record data
;==========================================================================================================================================================

Procedure DisplayRecordData()
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; Get the currently selected line number
    ;----------------------------------------------------------------------------------------------
    
    Program\CurrentLine = GetGadgetState(#Gadget_AppointmentMuncher_Appointments)
    
    ;----------------------------------------------------------------------------------------------
    ; Check if we got a selected line
    ;----------------------------------------------------------------------------------------------
    
    If Program\CurrentLine <> -1
      
      ;--------------------------------------------------------------------------------------------
      ; Get the record number from the selected line
      ;--------------------------------------------------------------------------------------------
      
      appDeleted.s = GetGadgetItemText(#Gadget_AppointmentMuncher_Appointments, Program\CurrentLine, Program\DeletedColumn)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If appDeleted <> "1"
        
        ;------------------------------------------------------------------------------------------
        ; Get the record number from the selected line
        ;------------------------------------------------------------------------------------------
        
        Program\CurrentRecord = GetGadgetItemText(#Gadget_AppointmentMuncher_Appointments, Program\CurrentLine, Program\RecordColumn)
        
        ;------------------------------------------------------------------------------------------
        ; Check if there was a record number to get
        ;------------------------------------------------------------------------------------------
        
        If Program\CurrentRecord <> ""
          
          ;----------------------------------------------------------------------------------------
          ; Setup the database query
          ;----------------------------------------------------------------------------------------
          
          QueryString.s = "SELECT * FROM Appointments "
          
          QueryString.s + "WHERE appRecord = '" + Program\CurrentRecord + "' "
          
          QueryString.s + "AND appDeleted <> '1'"
          
          ;----------------------------------------------------------------------------------------
          ; Minimise the amount of structure typing I have to do
          ;----------------------------------------------------------------------------------------
          
          With Appointment
            
            ;--------------------------------------------------------------------------------------
            ; Run the database query
            ;--------------------------------------------------------------------------------------
            
            If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)
              
              ;------------------------------------------------------------------------------------
              ; Fetch the next available database row
              ;------------------------------------------------------------------------------------
              
              While NextDatabaseRow(Program\DatabaseHandle)
                
                ;----------------------------------------------------------------------------------
                ; Retrieve the database information
                ;----------------------------------------------------------------------------------
                
                \appName        = KillQuote(GetDatabaseString(Program\DatabaseHandle,   0))
                
                \appDate        =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  1), "NormalDate")
                
                \appTime        =            GetDatabaseString(Program\DatabaseHandle,  2) 
                 
                \appAlarm       =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  3), "NormalDate")
                
                \appRepeat      =            GetDatabaseString(Program\DatabaseHandle,  4) 
                
                \appProtect     =            GetDatabaseString(Program\DatabaseHandle,  5) 
                
                \appCategory    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  6))
                
                \appStatus      =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  7))
                
                \appComment     =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  8))
                
                \appArchived    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  9))
                
                \appEmailed     =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 10))
                
                \appSound       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 11))
                
                \appDeleted     =            GetDatabaseString(Program\DatabaseHandle, 12) 
                
                \appRecord      =            GetDatabaseString(Program\DatabaseHandle, 13) 
                
                ;----------------------------------------------------------------------------------
                ; Go to the next row returned
                ;----------------------------------------------------------------------------------
                
              Wend
              
              ;------------------------------------------------------------------------------------
              ; Close off this database query
              ;------------------------------------------------------------------------------------
              
              FinishDatabaseQuery(Program\DatabaseHandle)
              
              ;------------------------------------------------------------------------------------
              ; Set the returned data to the gadgets
              ;------------------------------------------------------------------------------------
              
              Select \appRepeat
                  
                Case  "0" : RepeatTextToSet.s = "No Repeat"
                  
                Case  "1" : RepeatTextToSet.s = "Hourly"
                  
                Case  "2" : RepeatTextToSet.s = "Daily"
                  
                Case  "3" : RepeatTextToSet.s = "Weekly"
                  
                Case  "4" : RepeatTextToSet.s = "Monthly"
                  
                Case  "5" : RepeatTextToSet.s = "Yearly"
                  
              EndSelect

              SetGadgetText(#Gadget_AppointmentMuncher_Repeat,   RepeatTextToSet.s)
              
              Select \appProtect
                  
                Case  "0" : ProtectTextToSet.s = "Unlocked"
                  
                Case  "1" : ProtectTextToSet.s = "Locked"
                  
              EndSelect
              
              SetGadgetText(#Gadget_AppointmentMuncher_Protect,  ProtectTextToSet.s)
              
              SetGadgetText(#Gadget_AppointmentMuncher_Category, \appCategory)
              
              SetGadgetText(#Gadget_AppointmentMuncher_Status,   \appStatus)
              
              SetGadgetText(#Gadget_AppointmentMuncher_Comment,  \appComment)
              
              ;------------------------------------------------------------------------------------
              ; Tell the fake statusbar what appointment and record number this is
              ;------------------------------------------------------------------------------------
              
              SetInfoBarArea("Headings", "Appoint", \appName)
              
              SetInfoBarArea("Number",    "",       \appRecord)
              
              ;------------------------------------------------------------------------------------
              ; Oh oh, the database query has failed
              ;------------------------------------------------------------------------------------
              
            Else
              
              SetInfoBarArea("Headings", "Error", #InfoStringQueryFailed + DatabaseError(), "DisplayRecordData")
              
            EndIf
            
            ;--------------------------------------------------------------------------------------
            ; Turn off the structure alias type saver
            ;--------------------------------------------------------------------------------------
            
          EndWith
          
          ;----------------------------------------------------------------------------------------
          ; Oh dear, no database record number found on that line
          ;----------------------------------------------------------------------------------------
          
        Else
          
          SetInfoBarArea("Headings", "Error", #InfoStringNoRecord, "DisplayRecordData")
          
        EndIf
        
        ;------------------------------------------------------------------------------------------
        ; No need for this error message, just here for completeness
        ;------------------------------------------------------------------------------------------
        
      Else
        
        ;------------------------------------------------------------------------------------------
        ; Tell the fake statusbar what appointment and record number this is
        ;------------------------------------------------------------------------------------------
        
        SetGadgetText(#Gadget_AppointmentMuncher_Repeat,   "")
        
        SetGadgetText(#Gadget_AppointmentMuncher_Protect,  "")
        
        SetGadgetText(#Gadget_AppointmentMuncher_Category, "")
        
        SetGadgetText(#Gadget_AppointmentMuncher_Status,   "")
        
        SetGadgetText(#Gadget_AppointmentMuncher_Comment,  "")
        
        ;--------------------------------------------------------------------------------------
        ; Tell the fake statusbar what appointment and record number this is
        ;--------------------------------------------------------------------------------------
        
;       SetInfoBarArea("Headings", "Info", "No records found")
        
        SetInfoBarArea("Number",    "",    "?")      
        
        SetInfoBarArea("Headings", "Error", #InfoStringCannotShow, "DisplayRecordData")
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; No need for this error message, just here for completeness
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      ; SetInfoBarArea("Headings", "Error", #InfoStringNotSelected)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; Allow the alarm checker to run now
    ;----------------------------------------------------------------------------------------------
    
    Program\DontCheck = 0
    
    ;----------------------------------------------------------------------------------------------
    ; Let people know that the alarm checker is working so no other routines should run/be run
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 46
; FirstLine = 43
; Folding = -
; EnableXP