﻿;========================================================================================================================================
; Find appointments when the find button is pressed
;========================================================================================================================================

Procedure FindAppointmentsNow()
  
  ;------------------------------------------------------------------------------------------------
  ; Get the text that you want to search for
  ;------------------------------------------------------------------------------------------------
  
  SearchText.s  = GetGadgetText(#Gadget_AppointmentFind_Searchtext)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If SearchText.s <> ""
    
    ;----------------------------------------------------------------------------------------------
    ; Clear all items on the main appointment list
    ;----------------------------------------------------------------------------------------------
    
    ClearGadgetItems(#Gadget_AppointmentMuncher_Appointments)
    
    ;----------------------------------------------------------------------------------------------
    ; Clear the text from the search field
    ;----------------------------------------------------------------------------------------------
    
    SetGadgetText(#Gadget_AppointmentFind_Searchtext, "")
    
    ;----------------------------------------------------------------------------------------------
    ; Get the type of SQL search to be performed
    ;----------------------------------------------------------------------------------------------
    
    SearchType.s  = GetGadgetText(#Gadget_AppointmentFind_FindType)
    
    ;----------------------------------------------------------------------------------------------
    ; Alias the search type text to SQLite operators
    ;----------------------------------------------------------------------------------------------
    
    SearchCommand.s = SetSearchQuery(SearchType.s, SearchText.s)
    
    ;----------------------------------------------------------------------------------------------
    ; Find out which field should be searched
    ;----------------------------------------------------------------------------------------------
    
    SearchField.s = GetGadgetText(#Gadget_AppointmentFind_Searchfield)
    
    ;----------------------------------------------------------------------------------------------
    ; Work out the name of the field to be searched from the field text
    ;----------------------------------------------------------------------------------------------
    
    Select SearchField.s
        
      Case  "Appointment name"      : SearchField.s = "appName"
        
      Case  "Date to execute"       : SearchField.s = "appDate"
        
      Case  "Time of day"           : SearchField.s = "appTime"
        
      Case  "Alarm Set"             : SearchField.s = "appAlarm"
        
      Case  "Recurring date"        : SearchField.s = "appRepeat"
        
      Case  "Protected item"        : SearchField.s = "appProtect"
        
      Case  "Category type"         : SearchField.s = "appCategory"
        
      Case  "Status of appointment" : SearchField.s = "appStatus"
        
      Case  "Extra comments"        : SearchField.s = "appComment"
        
      Case  "Archived copy"         : SearchField.s = "appArchived"
        
      Case  "Appointment emailed"   : SearchField.s = "appEmailed"
        
      Case  "Sound file for alarm"  : SearchField.s = "appSound"
        
      Case  "Marked deleted or not" : SearchField.s = "appDeleted"
        
      Case  "Record number"         : SearchField.s = "appRecord"
        
    EndSelect

    ;----------------------------------------------------------------------------------------------
    ; Get the field on which the sort entries are to be returned
    ;----------------------------------------------------------------------------------------------
    
    SortField.s   = GetGadgetText(#Gadget_AppointmentFind_SortField)
    
    ;----------------------------------------------------------------------------------------------
    ; Work out the name of the field to be searched from the field text
    ;----------------------------------------------------------------------------------------------
    
    Select SortField.s
        
      Case  "Appointment name"      : SortField.s   = "appName"
        
      Case  "Date to execute"       : SortField.s   = "appDate"
        
      Case  "Time of day"           : SortField.s   = "appTime"
        
      Case  "Alarm Set"             : SortField.s   = "appAlarm"
        
      Case  "Recurring date"        : SortField.s   = "appRepeat"
        
      Case  "Protected item"        : SortField.s   = "appProtect"

      Case  "Category type"         : SortField.s   = "appCategory"
        
      Case  "Status of appointment" : SortField.s   = "appStatus"

      Case  "Extra comments"        : SortField.s   = "appComment"
        
      Case  "Archived copy"         : SearchField.s = "appArchived"
        
      Case  "Appointment emailed"   : SearchField.s = "appEmailed"
        
      Case  "Sound file for alarm"  : SearchField.s = "appSound"
        
      Case  "Marked deleted or not" : SortField.s   = "appDeleted"
        
      Case  "Record number"         : SortField.s   = "appRecord"
        
    EndSelect
    
    ;----------------------------------------------------------------------------------------------
    ; Find out if we are sorting up or down
    ;----------------------------------------------------------------------------------------------
    
    If GetGadgetState(#Gadget_AppointmentFind_oSortAcending) = 1
      
      SearchOrder.s = "ASC "
      
    ElseIf  GetGadgetState(#Gadget_AppointmentFind_oSortDescending) = 1
      
      SearchOrder.s = "DESC "
      
    Else
      
      SearchOrder.s = ""
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; Find out if we are going to limit the entries returned
    ;----------------------------------------------------------------------------------------------
    
    If GetGadgetState(#Gadget_AppointmentFind_cbQueryReturnLimit) = 1
      
      SearchLimit.s = "LIMIT " + GetGadgetText(#Gadget_AppointmentFind_sQueryReturnLimit)
      
    Else
      
      SearchLimit.s = ""
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    CloseFlag.i = GetGadgetState(#Gadget_AppointmentFind_cCloseOnFind)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    QueryString.s = "SELECT * FROM Appointments "         ; SELECT * FROM Appointments 
    
    QueryString.s + "WHERE " + SearchField.s    + " "     ; WHERE appName 
    
    QueryString.s + SearchCommand.s             + " "     ; LIKE '%doctor''s%'
    
    QueryString.s + "ORDER BY " + SortField.s   + " "     ; ORDER BY appDate
    
    QueryString.s + SearchOrder.s                         ; ASC
    
    QueryString.s + SearchLimit.s                         ; LIMIT 10
    
    ;Debug QueryString.s
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    ItemCounter.i = 0
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    With Appointment
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        While NextDatabaseRow(Program\DatabaseHandle)
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          \appName        = KillQuote(GetDatabaseString(Program\DatabaseHandle,   0))
          
          \appDate        =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  1), "NormalDate")
          
          \appTime        =            GetDatabaseString(Program\DatabaseHandle,  2) 
           
          \appAlarm       =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  3), "NormalDate")
          
          \appRepeat      =            GetDatabaseString(Program\DatabaseHandle,  4) 
          
          \appProtect     =            GetDatabaseString(Program\DatabaseHandle,  5) 
          
          \appCategory    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  6))
          
          \appStatus      =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  7))
          
          \appComment     =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  8))
          
          \appArchived    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  9))
          
          \appEmailed     =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 10))
          
          \appSound       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 11))
          
          \appDeleted     =            GetDatabaseString(Program\DatabaseHandle, 12) 
          
          \appRecord      =            GetDatabaseString(Program\DatabaseHandle, 13) 
          
          ;----------------------------------------------------------------------------------------
          ; Now, off to the display routine. It must set icons, data and nicely formatted dates
          ;----------------------------------------------------------------------------------------
          
          DisplayAppointmentString(ItemCounter.i, "AddMode")
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          ItemCounter.i + 1
          
          ;----------------------------------------------------------------------------------------
          ; Tell the user what is going on in the status bar 
          ;----------------------------------------------------------------------------------------
          
          SetInfoBarArea("Headings", "Info",  "Showing: " + KillQuote(\appName))
          
          SetInfoBarArea("Records",  "",        Str(ItemCounter.i))
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          ClearStructure(@Appointment, AppointmentData)
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        Wend
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        FinishDatabaseQuery(Program\DatabaseHandle)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        If CountGadgetItems(#Gadget_AppointmentMuncher_Appointments) <> 0
          
          SetGadgetState(#Gadget_AppointmentMuncher_Appointments, 0)
          
          PostEvent(#PB_Event_Gadget, #Window_AppointmentMuncher, #Gadget_AppointmentMuncher_Appointments, #PB_EventType_Change)
          
        Else
          
          ClearGadgetItems(#Gadget_AppointmentMuncher_Appointments)
          
          SetGadgetText(#Gadget_AppointmentMuncher_Repeat,   "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Protect,  "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Category, "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Status,   "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Comment,  "")
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;-----------------------------------------------------------------------------------------
          
          SetInfoBarArea("Headings", "Info", "No records found for the [" + SearchText.s + " query.")
          
          SetInfoBarArea("Number",    "",    "?")
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        EndIf
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      Else
        
        SetInfoBarArea("Headings", "Error", #InfoStringQueryFailed + DatabaseError())
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndWith

    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    If CloseFlag.i
      
      CloseMyWindow(#Window_AppointmentFind)
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Info",  #InfoStringNoStringToFind)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 114
; FirstLine = 111
; Folding = -
; EnableXP