﻿;==========================================================================================================================================================
; List all appointments at startup
;==========================================================================================================================================================

Procedure ListAppointments(DisplayType.s)
  
  ;------------------------------------------------------------------------------------------------
  ; Check if the alarm checker is running first
  ;------------------------------------------------------------------------------------------------
  
  If Program\DontCheck <> 1
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DontCheck = 1
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    ClearGadgetItems(#Gadget_AppointmentMuncher_Appointments)
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Select DisplayType.s
        
      Case "Selected"   ; Show dates for a date selected on the calendar gadget
        
                          DateString.s = IsoDate(FormatDate("%dd/%mm/%yyyy", GetGadgetState(#Gadget_AppointmentMuncher_DatePicker)), "IsoDate")
        
                          QueryString.s = "SELECT * FROM Appointments WHERE appDate = '" + DateString.s + "'"
        
      Case "All"        : QueryString.s = "SELECT * FROM Appointments"
        
      Case "Active"     : QueryString.s = "Select * FROM Appointments WHERE appStatus = 'Active appointment' AND appDeleted <> '1' AND appProtect <> '1'"
        
      Case "Past"       : QueryString.s = "SELECT * FROM Appointments WHERE appDate < '"  + Program\ISODate + "' AND appDeleted <> '1' AND appProtect <> '1'"
        
      Case "Today"      : QueryString.s = "SELECT * FROM Appointments WHERE appDate = '"  + Program\ISODate + "' AND appDeleted <> '1' AND appProtect <> '1'"
        
      Case "Future"     : QueryString.s = "SELECT * FROM Appointments WHERE appDate > '"  + Program\ISODate + "' AND appDeleted <> '1' AND appProtect <> '1'"
        
      Case "Finished"   : QueryString.s = "SELECT * FROM Appointments WHERE appStatus = 'Finished appointment' AND appDeleted <> '1' AND appProtect <> '1'"
        
      Case "Deleted"    : QueryString.s = "SELECT * FROM Appointments WHERE appDeleted = '1' AND appProtect <> '1'"
        
      Case "Protected"  : QueryString.s = "SELECT * FROM Appointments WHERE appProtect = '1'"
        
    EndSelect
  
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    ItemCounter.i = 0
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    With Appointment
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If DatabaseQuery(Program\DatabaseHandle,  QueryString.s)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        While NextDatabaseRow(Program\DatabaseHandle)
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          \appName        = KillQuote(GetDatabaseString(Program\DatabaseHandle,   0))
          
          \appDate        =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  1), "NormalDate")
          
          \appTime        =            GetDatabaseString(Program\DatabaseHandle,  2) 
           
          \appAlarm       =    IsoDate(GetDatabaseString(Program\DatabaseHandle,  3), "NormalDate")
          
          \appRepeat      =            GetDatabaseString(Program\DatabaseHandle,  4) 
          
          \appProtect     =            GetDatabaseString(Program\DatabaseHandle,  5) 
          
          \appCategory    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  6))
          
          \appStatus      =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  7))
          
          \appComment     =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  8))
          
          \appArchived    =  KillQuote(GetDatabaseString(Program\DatabaseHandle,  9))
          
          \appEmailed     =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 10))
          
          \appSound       =  KillQuote(GetDatabaseString(Program\DatabaseHandle, 11))
          
          \appDeleted     =            GetDatabaseString(Program\DatabaseHandle, 12) 
          
          \appRecord      =            GetDatabaseString(Program\DatabaseHandle, 13) 
          
          ;----------------------------------------------------------------------------------------
          ; Now, off to the display routine. It must set icons, data and nicely formatted dates
          ;----------------------------------------------------------------------------------------
          
          DisplayAppointmentString(ItemCounter.i, "AddMode")
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          ItemCounter.i + 1
          
          ;----------------------------------------------------------------------------------------
          ; Tell the user what is going on in the status bar 
          ;----------------------------------------------------------------------------------------
          
          SetInfoBarArea("Headings", "Info",  "Showing: " + KillQuote(\appName))
          
          SetInfoBarArea("Records",  "",        Str(ItemCounter.i))
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
          ClearStructure(@Appointment, AppointmentData)
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;----------------------------------------------------------------------------------------
          
        Wend
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        FinishDatabaseQuery(Program\DatabaseHandle)
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
        If CountGadgetItems(#Gadget_AppointmentMuncher_Appointments) <> 0
          
          SetGadgetState(#Gadget_AppointmentMuncher_Appointments, 0)
          
          PostEvent(#PB_Event_Gadget, #Window_AppointmentMuncher, #Gadget_AppointmentMuncher_Appointments, #PB_EventType_Change)
          
        Else
          
          ClearGadgetItems(#Gadget_AppointmentMuncher_Appointments)
          
          SetGadgetText(#Gadget_AppointmentMuncher_Repeat,   "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Protect,  "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Category, "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Status,   "")
          
          SetGadgetText(#Gadget_AppointmentMuncher_Comment,  "")
            
          ;----------------------------------------------------------------------------------------
          ; Tell the fake statusbar what appointment and record number this is
          ;----------------------------------------------------------------------------------------
          
          If DisplayType.s = "Today"
            
            DisplayType.s = "Today's"
            
          EndIf
          
          SetInfoBarArea("Headings", "Info", "No records found for the [" + DisplayType.s + " appointments] option setting")
          
          SetInfoBarArea("Number",    "",    "?")
          
          ;----------------------------------------------------------------------------------------
          ; 
          ;-----------------------------------------------------------------------------------------
          
        EndIf
        
        ;------------------------------------------------------------------------------------------
        ; 
        ;------------------------------------------------------------------------------------------
        
      Else
        
        SetInfoBarArea("Headings", "Error", #InfoStringQueryFailed + DatabaseError(), "ListAppointments")
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    EndWith
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
    Program\DontCheck = 0
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
      
    SetInfoBarArea("Headings",  "Warn",  #InfoStringAlarmRunning)
      
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure



; IDE Options = PureBasic 5.23 LTS (Windows - x86)
; CursorPosition = 109
; FirstLine = 72
; Folding = -
; EnableXP