﻿;==========================================================================================================================================================
; Import an XML file of appointment entries
;==========================================================================================================================================================

Procedure ImportAppointments()
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  ImportFile.s = OpenFileRequester("Import file from", Program\TemporaryDirectory, "XML file (*.xml)|*.xml", 0)
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
  If ImportFile.s
    
    ;----------------------------------------------------------------------------------------------
    ; Did we get a file handle?
    ;----------------------------------------------------------------------------------------------
    
    ImportFileId.i = LoadXML(#PB_Any, ImportFile.s)
    
    ;----------------------------------------------------------------------------------------------
    ; Did we get a file handle?
    ;----------------------------------------------------------------------------------------------
    
    If ImportFileId.i
      
      ;--------------------------------------------------------------------------------------------
      ; Start the SQL command transactions to speed up additions to the database
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate(Program\DatabaseHandle, "Begin Transaction")
      
      ;--------------------------------------------------------------------------------------------
      ; Get the main node of the XML document
      ;--------------------------------------------------------------------------------------------
      
      *MainNode = MainXMLNode(ImportFileId.i)
      
      ;--------------------------------------------------------------------------------------------
      ; Parse the XML file
      ;--------------------------------------------------------------------------------------------
      
      ProcessAppointmentNode(*MainNode, CurrentSublevel.i)
      
      ;--------------------------------------------------------------------------------------------
      ; Commit All the SQL transactions to the database
      ;--------------------------------------------------------------------------------------------
      
      DatabaseUpdate(Program\DatabaseHandle, "Commit")
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      SetInfoBarArea("Headings",  "Info",  #InfoStringImportFileDone)
      
      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
      If Setup\StartupDates
        
        Select Setup\StartupDates
            
          Case 0  : GadgetNumber.i = #Gadget_AppointmentMuncher_oAll
            
          Case 1  : GadgetNumber.i = #Gadget_AppointmentMuncher_oActive
            
          Case 2  : GadgetNumber.i = #Gadget_AppointmentMuncher_oPast
            
          Case 3  : GadgetNumber.i = #Gadget_AppointmentMuncher_oToday
            
          Case 4  : GadgetNumber.i = #Gadget_AppointmentMuncher_oFuture
            
          Case 5  : GadgetNumber.i = #Gadget_AppointmentMuncher_oFinished
            
          Case 6  : GadgetNumber.i = #Gadget_AppointmentMuncher_oDeleted
            
          Default
            
        EndSelect
        
        SetGadgetState(GadgetNumber.i, 1)
        
      Else
        
        GadgetNumber.i  = #Gadget_AppointmentMuncher_oAll
        
        SetGadgetState(GadgetNumber.i, 1)
        
      EndIf
      
      ;--------------------------------------------------------------------------------------------
      ; Show the appointment names for the selected option set of dates by triggering a change event on;
      ; it.
      ;--------------------------------------------------------------------------------------------
      
      PostEvent(#PB_Event_Gadget, #Window_AppointmentMuncher, GadgetNumber.i, #PB_EventType_Change)

      ;--------------------------------------------------------------------------------------------
      ; 
      ;--------------------------------------------------------------------------------------------
      
    Else
      
      SetInfoBarArea("Headings",  "Error",  #InfoStringImportFileFail, "ImportAppointments")
      
    EndIf
    
    ;----------------------------------------------------------------------------------------------
    ; 
    ;----------------------------------------------------------------------------------------------
    
  Else
    
    SetInfoBarArea("Headings",  "Info",  #InfoStringImportFileCancel)
    
  EndIf
  
  ;------------------------------------------------------------------------------------------------
  ; 
  ;------------------------------------------------------------------------------------------------
  
EndProcedure


; IDE Options = PureBasic 5.22 LTS (Windows - x86)
; CursorPosition = 109
; FirstLine = 98
; Folding = -
; EnableXP